"""

  fs.tests.test_fs:  testcases for basic FS implementations

"""

from fs.tests import FSTestCases, ThreadingTestCases
from fs.path import *
from fs import errors

import unittest

import os
import sys
import shutil
import tempfile


from fs import osfs
class TestOSFS(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp(u"fstest")
        self.fs = osfs.OSFS(self.temp_dir)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        self.fs.close()

    def check(self, p):
        return os.path.exists(os.path.join(self.temp_dir, relpath(p)))

    def test_invalid_chars(self):
        super(TestOSFS, self).test_invalid_chars()

        self.assertRaises(errors.InvalidCharsInPathError, self.fs.open, 'invalid\0file', 'wb')
        self.assertFalse(self.fs.isvalidpath('invalid\0file'))
        self.assert_(self.fs.isvalidpath('validfile'))
        self.assert_(self.fs.isvalidpath('completely_valid/path/foo.bar'))


class TestSubFS(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp(u"fstest")
        self.parent_fs = osfs.OSFS(self.temp_dir)
        self.parent_fs.makedir("foo/bar", recursive=True)
        self.fs = self.parent_fs.opendir("foo/bar")

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        self.fs.close()

    def check(self, p):
        p = os.path.join("foo/bar", relpath(p))
        full_p = os.path.join(self.temp_dir, p)
        return os.path.exists(full_p)


from fs import memoryfs
class TestMemoryFS(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.fs = memoryfs.MemoryFS()


from fs import mountfs
class TestMountFS(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.mount_fs = mountfs.MountFS()
        self.mem_fs = memoryfs.MemoryFS()
        self.mount_fs.mountdir("mounted/memfs", self.mem_fs)
        self.fs = self.mount_fs.opendir("mounted/memfs")

    def tearDown(self):
        self.fs.close()

    def check(self, p):
        return self.mount_fs.exists(pathjoin("mounted/memfs", relpath(p)))

class TestMountFS_atroot(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.mem_fs = memoryfs.MemoryFS()
        self.fs = mountfs.MountFS()
        self.fs.mountdir("", self.mem_fs)

    def tearDown(self):
        self.fs.close()

    def check(self, p):
        return self.mem_fs.exists(p)

class TestMountFS_stacked(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.mem_fs1 = memoryfs.MemoryFS()
        self.mem_fs2 = memoryfs.MemoryFS()
        self.mount_fs = mountfs.MountFS()
        self.mount_fs.mountdir("mem", self.mem_fs1)
        self.mount_fs.mountdir("mem/two", self.mem_fs2)
        self.fs = self.mount_fs.opendir("/mem/two")

    def tearDown(self):
        self.fs.close()

    def check(self, p):
        return self.mount_fs.exists(pathjoin("mem/two", relpath(p)))


from fs import tempfs
class TestTempFS(unittest.TestCase,FSTestCases,ThreadingTestCases):

    def setUp(self):
        self.fs = tempfs.TempFS()

    def tearDown(self):
        td = self.fs._temp_dir
        self.fs.close()
        self.assert_(not os.path.exists(td))

    def check(self, p):
        td = self.fs._temp_dir
        return os.path.exists(os.path.join(td, relpath(p)))

    def test_invalid_chars(self):
        super(TestTempFS, self).test_invalid_chars()

        self.assertRaises(errors.InvalidCharsInPathError, self.fs.open, 'invalid\0file', 'wb')
        self.assertFalse(self.fs.isvalidpath('invalid\0file'))
        self.assert_(self.fs.isvalidpath('validfile'))
        self.assert_(self.fs.isvalidpath('completely_valid/path/foo.bar'))



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from traits.api import HasTraits, Instance, List, Unicode
from traitsui.api import Item, TableEditor, View
from traitsui.menu import OKCancelButtons
from traitsui.table_column import ObjectColumn


class _User(HasTraits):
    """This represents the user model."""

    #### '_User' interface ####################################################

    # The user name.
    name = Unicode

    # The user description.
    description = Unicode


class _UsersView(HasTraits):
    """This represents the view used to select a user."""

    #### '_UsersView' interface ###############################################

    # The list of users to select from.
    model = List(_User)

    # The selected user.
    selection = Instance(_User)

    # The editor used by the view.
    table_editor = TableEditor(columns=[ObjectColumn(name='name'),
                    ObjectColumn(name='description')],
            selected='selection', sort_model=True, configurable=False)

    # The default view.
    traits_view = View(Item('model', show_label=False, editor=table_editor),
            title="Select a User", style='readonly', kind='modal',
            buttons=OKCancelButtons)


def select_user(users):
    """Return a single user from the given list of users."""

    # Construct the model.
    model = [_User(name=name, description=description)
            for name, description in users]

    # Construct the view.
    view = _UsersView(model=model)

    if view.configure_traits() and view.selection is not None:
        user = view.selection.name, view.selection.description
    else:
        user = '', ''

    return user



#------------------------------------------------------------------------------
# Copyright (c) 2008, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the GPL v2
# license.
#
# Author: Vibha Srinivasan <vibha@enthought.com>
#------------------------------------------------------------------------------

""" Generates a file containing definitions for all of the editors defined in
the WX backend.
"""

import os
import glob
import sys
from traitsui.api import Editor
from traitsui.editor_factory import EditorFactory


def gen_editor_definitions(target_filename):
    """ Generates a file containing definitions for all of the editors
    defined in the Qt backend.
    """

    target_file = open(target_filename, 'w')
    dirpath = os.path.dirname(os.path.abspath(__file__))
    # Find all the files which define a TraitsUIEditor
    editor_files = []
    for (root, dirs, files) in os.walk(dirpath):
        if '.svn' in dirs:
            dirs.remove('.svn')
        editor_files.extend(glob.glob(os.path.join(root, '*_editor.py')))

    for absfilename in editor_files:
        (dirname, filename) = (os.path.dirname(absfilename),
                               os.path.basename(absfilename).rstrip('.py'))
        import_path = 'traitsui.wx' + \
            dirname.replace(dirpath, '').replace(os.sep, '.') +\
            '.' + filename
        __import__(import_path)
        module = sys.modules[import_path]
        class_names = []
        for name in dir(module):
            try:
                if issubclass(getattr(module, name), EditorFactory) and \
                        name not in ['EditorFactory', 'BasicEditorFactory']:
                    class_names.append(name)
                elif issubclass(getattr(module, name), Editor) and \
                        name != 'Editor':
                    class_names.append(name)
            except:
                try:
                    if isinstance(getattr(module, name), EditorFactory) or \
                            isinstance(getattr(module, name), Editor):
                        class_names.insert(0, name)
                except:
                    pass

        if len(class_names) > 0:
            # FIXME: Is there a better way to sort these names?
            if 'ToolkitEditorFactory' in class_names:
                class_name = 'ToolkitEditorFactory'
            else:
                class_name = ''.join([name.capitalize() for name in
                                      filename.split('_')])
                if class_name not in class_names:
                    class_name = class_names[0]
            function = "def %(filename)s(*args, **traits):" % locals()
            target_file.write(function)
            target_file.write('\n')
            func_code = ' ' * 4 + \
                "import %(import_path)s as editor" % locals() + '\n'
            func_code += ' ' * 4 + \
                "return editor.%(class_name)s(*args, **traits)" % locals()
            target_file.write(func_code)
            target_file.write('\n\n')

    target_file.close()



"""
fs.xattrs
=========

Extended attribute support for FS

This module defines a standard interface for FS subclasses that want to
support extended file attributes, and a WrapFS subclass that can simulate
extended attributes on top of an ordinary FS.

FS instances offering extended attribute support must provide the following
methods:

  * ``getxattr(path,name)`` Get the named attribute for the given path, or None if it does not exist
  * ``setxattr(path,name,value)`` Set the named attribute for the given path to the given value
  * ``delxattr(path,name)`` Delete the named attribute for the given path, raising KeyError if it does not exist
  * ``listxattrs(path)`` Iterate over all stored attribute names for the given path

If extended attributes are required by FS-consuming code, it should use the
function 'ensure_xattrs'. This will interrogate an FS object to determine
if it has native xattr support, and return a wrapped version if it does not.
"""

import sys
try:
    import cPickle as pickle
except ImportError:
    import pickle

from fs.path import *
from fs.errors import *
from fs.wrapfs import WrapFS
from fs.base import synchronize


def ensure_xattrs(fs):
    """Ensure that the given FS supports xattrs, simulating them if required.

    Given an FS object, this function returns an equivalent FS that has support
    for extended attributes.  This may be the original object if they are
    supported natively, or a wrapper class is they must be simulated.
    
    :param fs: An FS object that must have xattrs
    """
    try:
        #  This attr doesn't have to exist, None should be returned by default
        fs.getxattr("/","testing-xattr")
        return fs
    except (AttributeError,UnsupportedError):
        return SimulateXAttr(fs)


class SimulateXAttr(WrapFS):
    """FS wrapper class that simulates xattr support.

    The following methods are supplied for manipulating extended attributes:
        * listxattrs:    list all extended attribute names for a path
        * getxattr:  get an xattr of a path by name
        * setxattr:  set an xattr of a path by name
        * delxattr:  delete an xattr of a path by name

    For each file in the underlying FS, this class maintains a corresponding 
    '.xattrs.FILENAME' file containing its extended attributes.  Extended
    attributes of a directory are stored in the file '.xattrs' within the
    directory itself.
    """

    def _get_attr_path(self, path, isdir=None):
        """Get the path of the file containing xattrs for the given path."""
        if isdir is None:
            isdir = self.wrapped_fs.isdir(path)
        if isdir:
            attr_path = pathjoin(path, '.xattrs')
        else:
            dir_path, file_name = pathsplit(path)
            attr_path = pathjoin(dir_path, '.xattrs.'+file_name)
        return attr_path

    def _is_attr_path(self, path):
        """Check whether the given path references an xattrs file."""
        _,name = pathsplit(path)
        if name.startswith(".xattrs"):
            return True
        return False

    def _get_attr_dict(self, path):
        """Retrieve the xattr dictionary for the given path."""
        attr_path = self._get_attr_path(path)
        if self.wrapped_fs.exists(attr_path):
            try:
                return pickle.loads(self.wrapped_fs.getcontents(attr_path))
            except EOFError:
                return {}
        else:
            return {}

    def _set_attr_dict(self, path, attrs):
        """Store the xattr dictionary for the given path."""
        attr_path = self._get_attr_path(path)
        self.wrapped_fs.setcontents(attr_path, pickle.dumps(attrs))

    @synchronize
    def setxattr(self, path, key, value):
        """Set an extended attribute on the given path."""
        if not self.exists(path):
            raise ResourceNotFoundError(path)
        key = unicode(key)
        attrs = self._get_attr_dict(path)
        attrs[key] = str(value)
        self._set_attr_dict(path, attrs)

    @synchronize
    def getxattr(self, path, key, default=None):
        """Retrieve an extended attribute for the given path."""
        if not self.exists(path):
            raise ResourceNotFoundError(path)
        attrs = self._get_attr_dict(path)
        return attrs.get(key, default)

    @synchronize
    def delxattr(self, path, key):
        if not self.exists(path):
            raise ResourceNotFoundError(path)
        attrs = self._get_attr_dict(path)
        try:
            del attrs[key]
        except KeyError:
            pass
        self._set_attr_dict(path, attrs)

    @synchronize
    def listxattrs(self,path):
        """List all the extended attribute keys set on the given path."""
        if not self.exists(path):
            raise ResourceNotFoundError(path)
        return self._get_attr_dict(path).keys()

    def _encode(self,path):
        """Prevent requests for operations on .xattr files."""
        if self._is_attr_path(path):
            raise PathError(path,msg="Paths cannot contain '.xattrs': %(path)s")
        return path

    def _decode(self,path):
        return path

    def listdir(self,path="",*args,**kwds):
        """Prevent .xattr from appearing in listings."""
        entries = self.wrapped_fs.listdir(path,*args,**kwds)
        return [e for e in entries if not self._is_attr_path(e)]

    def ilistdir(self,path="",*args,**kwds):
        """Prevent .xattr from appearing in listings."""
        for e in self.wrapped_fs.ilistdir(path,*args,**kwds):
            if not self._is_attr_path(e):
                yield e

    def remove(self,path):
        """Remove .xattr when removing a file."""
        attr_file = self._get_attr_path(path,isdir=False)
        self.wrapped_fs.remove(path)
        try:
            self.wrapped_fs.remove(attr_file)
        except ResourceNotFoundError:
            pass

    def removedir(self,path,recursive=False,force=False):
        """Remove .xattr when removing a directory."""
        try:
            self.wrapped_fs.removedir(path,recursive=recursive,force=force)
        except DirectoryNotEmptyError:
            #  The xattr file could block the underlying removedir().
            #  Remove it, but be prepared to restore it on error.
            if self.listdir(path) != []:
                raise
            attr_file = self._get_attr_path(path,isdir=True)
            attr_file_contents = self.wrapped_fs.getcontents(attr_file)
            self.wrapped_fs.remove(attr_file)
            try:
                self.wrapped_fs.removedir(path,recursive=recursive)
            except FSError:
                self.wrapped_fs.setcontents(attr_file,attr_file_contents)
                raise

    def copy(self,src,dst,**kwds):
        """Ensure xattrs are copied when copying a file."""
        self.wrapped_fs.copy(self._encode(src),self._encode(dst),**kwds)
        s_attr_file = self._get_attr_path(src)
        d_attr_file = self._get_attr_path(dst)
        try:
            self.wrapped_fs.copy(s_attr_file,d_attr_file,overwrite=True)
        except ResourceNotFoundError,e:
            pass

    def move(self,src,dst,**kwds):
        """Ensure xattrs are preserved when moving a file."""
        self.wrapped_fs.move(self._encode(src),self._encode(dst),**kwds)
        s_attr_file = self._get_attr_path(src)
        d_attr_file = self._get_attr_path(dst)
        try:
            self.wrapped_fs.move(s_attr_file,d_attr_file,overwrite=True)
        except ResourceNotFoundError:
            pass

 



import numpy as np
import pytest
from .utilities import twenty_one_triangles


# used to parametrize tests for both methods
try:
    import cell_tree2d #noqa
    methods = ['simple', 'celltree']
except:
    # no cell tree -- only test simple
    methods = ['simple']


@pytest.mark.parametrize("method", methods)
def test_single(method):
    ugrid = twenty_one_triangles()
    face = ugrid.locate_faces((4, 6.5), method)
    assert face == 6


@pytest.mark.parametrize("method", methods)
def test_multi(method):
    ugrid = twenty_one_triangles()
    face = ugrid.locate_faces(np.array(((4, 6.5), (7, 2))), method)
    assert (face == np.array((6, 0))).all()


@pytest.mark.parametrize("method", methods)
def test_oob(method):
    ugrid = twenty_one_triangles()
    face = ugrid.locate_faces((0, 0), method)
    assert face == -1
    face = 0
    face = ugrid.locate_faces(np.array(((0, 0),)), method)
    assert np.array_equal(face, np.array((-1, )))






#-------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/21/2004
#
#-------------------------------------------------------------------------

""" Defines a text editor which displays a text field and maintains a history
    of previously entered values.
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------

from traits.api \
    import Any, on_trait_change

from pyface.timer.api \
    import do_later

from editor \
    import Editor

from history_control \
    import HistoryControl

#-------------------------------------------------------------------------
#  '_HistoryEditor' class:
#-------------------------------------------------------------------------


class _HistoryEditor(Editor):
    """ Simple style text editor, which displays a text field and maintains a
        history of previously entered values, the maximum number of which is
        specified by the 'entries' trait of the HistoryEditor factory.
    """

    #-------------------------------------------------------------------------
    #  Trait definitions:
    #-------------------------------------------------------------------------

    # The history control:
    history = Any

    #-------------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #-------------------------------------------------------------------------

    def init(self, parent):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self.history = history = HistoryControl(
            value=self.value,
            entries=self.factory.entries,
            auto_set=self.factory.auto_set)
        self.control = history.create_control(parent)

        self.set_tooltip()

    def dispose(self):
        """ Disposes of the contents of an editor.
        """
        self.history.dispose()
        self.history = None

        super(_HistoryEditor, self).dispose()

    #-------------------------------------------------------------------------
    #  Handles the user entering input data in the edit control:
    #-------------------------------------------------------------------------

    @on_trait_change('history:value')
    def _value_changed(self, value):
        """ Handles the history object's 'value' trait being changed.
        """
        if not self._dont_update:
            history = self.history
            try:
                self._dont_update = True
                self.value = history.value
                history.error = False
            except:
                history.error = True

            do_later(self.set, _dont_update=False)

    #-------------------------------------------------------------------------
    #  Updates the editor when the object trait changes external to the editor:
    #-------------------------------------------------------------------------

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        if not self._dont_update:
            self._dont_update = True
            self.history.value = self.value
            self.history.error = False
            self._dont_update = False

    #-------------------------------------------------------------------------
    #  Handles an error that occurs while setting the object's trait value:
    #-------------------------------------------------------------------------

    def error(self, excp):
        """ Handles an error that occurs while setting the object's trait value.
        """
        pass

    #-- UI preference save/restore interface ---------------------------------

    #-------------------------------------------------------------------------
    #  Restores any saved user preference information associated with the
    #  editor:
    #-------------------------------------------------------------------------

    def restore_prefs(self, prefs):
        """ Restores any saved user preference information associated with the
            editor.
        """
        self.history.history = \
            prefs.get('history', [])[: self.factory.entries]

    #-------------------------------------------------------------------------
    #  Returns any user preference information associated with the editor:
    #-------------------------------------------------------------------------

    def save_prefs(self):
        """ Returns any user preference information associated with the editor.
        """
        # If the view closed successfully, try to update the history with the
        # current value:
        if self.ui.result:
            self._dont_update = True
            self.history.set_value(self.value)
            self._dont_update = False

        return {'history': self.history.history[:]}

# EOF #########################################################################



import sys
import random
import time
import zlib
import base64
import cStringIO
import math
import bisect

import wx
from wx.lib.agw.rulerctrl import RulerCtrl, TimeFormat, IntFormat
import wx.lib.scrolledpanel as scrolled

import logging
log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)

DateFormat = 6
MonthFormat = 7

time_steps = [minute * 60 for minute in [1, 2, 5, 10, 15, 20, 30]]
time_steps.extend([hour * 3600 for hour in [1, 2, 4, 6, 12]])
time_steps.extend([day * 86400 for day in [1, 2, 3, 7, 14, 28]])

def time_step(self, visible_pixels, total_pixels, low_value, high_value):
    time_range = high_value - low_value
    seconds_per_pixel = time_range / visible_pixels
    step = time_steps[min(bisect.bisect(time_steps, abs(seconds_per_pixel)), len(time_steps) - 1)]
    print time_steps
    print step
    return step


class BitSink(object):
    def __len__(self):
        return sys.maxint

    def __getitem__(self, i):
        return 0

    def __setitem__(self, i, val):
        return


class LabeledRuler(RulerCtrl):
    def __init__(self, *args, **kwargs):
        RulerCtrl.__init__(self, *args, **kwargs)
        self.common_init()

    def common_init(self):
        self._marks = {}
        self._mark_pen = wx.Pen(wx.RED)
        self._selected_mark_pen = wx.Pen(wx.RED, 2)
        self._pixel_hit_distance = 3
        self._highlight = wx.Colour(100, 200, 230)
        self.selected_ranges = []
        self.visible_range = (0,0)
        self.mark_length = 10  # pixels
        self.selected_mark_length = 18  # pixels
        self.use_leftmost = False

    @property
    def has_selection(self):
        return len(self.selected_ranges) > 0

    def position_to_value(self, pos):
        """Pixel position to data point value
        """
        perc = float(min(pos - self._left, self._length)) / self._length
        value = ((self._max - self._min)*perc) + self._min
        #print "position_to_value", value, perc, pos, self._length
        return value

    def value_to_position(self, value, clamp=False):
        """Data point value to pixel position
        """
        perc = (value - self._min)/(self._max - self._min)
        if perc >= 0.0 and perc <= 1.0:
            pos = perc * self._length
        elif clamp:
            pos = 0 if perc < 0.0 else self._length
        else:
            pos = None
        #print "value_to_position", value, perc, pos, self._length
        return pos

    def value_to_visible_position(self, value):
        pos = self.value_to_position(value)
        if pos is not None:
            if pos < self.visible_range[0] or pos > self.visible_range[1]:
                pos = None
        return pos

    def SetRange(self, minVal, maxVal):
        # make sure it's floating point! Otherwise you get lots of zeros
        # calculating position_to_value & value_to_position.
        minVal = float(minVal)
        maxVal = float(maxVal)
        if self._min != minVal or self._max != maxVal:
            self._min = minVal
            self._max = maxVal
            self.Invalidate()

    def Invalidate(self):
        self._valid = False
        self._length = self._right - self._left
        self._majorlabels = []
        self._minorlabels = []
        self.Refresh()

    def clear_marks(self):
        self._marks = {}

    def set_mark(self, value, data):
        self._marks[float(value)] = data

    def draw_mark(self, dc, pos, selected=False):
        if selected:
            length = self.selected_mark_length
            dc.SetPen(self._selected_mark_pen)
        else:
            length = self.mark_length
            dc.SetPen(self._mark_pen)
        dc.DrawLine(self._left + pos, self._bottom - length,
            self._left + pos, self._bottom)

    def all_marks(self):
        return set(self._marks.values())

    def marks_within_range(self, r):
        inside = []
        low, hi = r
        if low > hi:
            low, hi = hi, low
        for value, data in self._marks.iteritems():
            if value >= low and value <= hi:
                inside.append(data)
        return inside

    def marks_in_selection(self):
        total = set()
        for r in self.selected_ranges:
            inside = self.marks_within_range(r)
            total.update(inside)
        return total

    def marks_to_display_as_selected(self):
        # hook to override in subclass to add marks to the selection
        return self.marks_in_selection()

    def get_visible_range(self):
        panel = self.GetParent()
        x, _ = panel.GetViewStart()
        return x, x + panel.GetSize()[0] - 1

    def Draw(self, dc):
        if not self._valid:
            LabeledRuler.Update(self, dc)

        dc.SetBrush(wx.Brush(self._background))
        dc.SetPen(self._tickpen)
        dc.SetTextForeground(self._textcolour)

        r = self.GetClientRect()
        x, y = self.GetViewStart()

        dc.SetBrush(wx.Brush(self._highlight))
        dc.SetPen(wx.TRANSPARENT_PEN)
        for left, right in self.selected_ranges:
            if right < left:
                left, right = right, left
            r.SetLeft(self.value_to_position(left, True) - x)
            r.SetRight(self.value_to_position(right, True) - x)
            dc.DrawRectangleRect(r)

        dc.SetBrush(wx.Brush(self._background))
        dc.SetPen(self._tickpen)
        dc.DrawLine(self._left, self._bottom-1, self._right+1, self._bottom-1)

        dc.SetFont(self._majorfont)

        left, right = self.get_visible_range()

        leftmost_major_pos = right
        leftmost_clear_zone = 0
        leftmost_label = None

        for label in self._majorlabels:
            pos = label.pos

            if pos < left - 100 or pos > right + 100:
                #print "skipping major: %s" % label.text, pos, left, right
                continue
            
            #print "plotting major: %s" % label.text, pos, left, right

            if pos < leftmost_major_pos:
                leftmost_major_pos = pos
                leftmost_label = label
                #print "leftmost", pos, left, self._left

            dc.DrawLine(self._left + pos - x, self._bottom - 5,
                self._left + pos - x, self._bottom)
            
            if label.text != "":
                dc.DrawText(label.text, self._left + label.lx - x, label.ly - 4)

        if self.use_leftmost:
            text = leftmost_label.text
            strw, strh = dc.GetTextExtent(text)

            if leftmost_major_pos > left + strw + 4:
                print text, left, self._left
                dc.DrawText(text, self._left + left + 1 - x, self._bottom - strh - 10)
                leftmost_clear_zone = int(left + (strw * 1.8))
        
        dc.SetFont(self._minorfont)

        for label in self._minorlabels:
            pos = label.pos

            if pos < left - 100 or pos > right + 100:
                #print "skipping major: %s" % label.text, pos, left, right
                continue

            dc.DrawLine(self._left + pos - x, self._bottom - 3,
                self._left + pos - x, self._bottom)
            
            if label.lx == 0:
                continue

            if label.text != "" or (not self.use_leftmost or label.lx >= leftmost_clear_zone):
                dc.DrawText(label.text, label.lx - x, label.ly)

        for indicator in self._indicators:
            indicator.Draw(dc)

        dc.SetBrush(wx.Brush(self._background))
        dc.SetPen(self._mark_pen)

        selected = self.marks_to_display_as_selected()
        for value, data in self._marks.iteritems():
            pos = self.value_to_position(value)
            if pos is None:
                # skip offscreen marks
                continue
            self.draw_mark(dc, pos - x, data in selected)

    def hit_test(self, mouse_pos):
        if mouse_pos < 0 or mouse_pos >= self._length:
            return None

        x, y = self.GetViewStart()
        mouse_pos += x
        for value, data in self._marks.iteritems():
            pos = self.value_to_position(value)
            if pos is None:
                # skip offscreen marks
                continue
            if abs(mouse_pos - pos) < self._pixel_hit_distance:
                return data
        return None

    def LabelString(self, d, major=None):
        if self._format == TimeFormat:
            if self._timeformat == DateFormat:
                t = time.gmtime(d)
                s = time.strftime("%y%m%d %H%M%S", t)
            elif self._timeformat == MonthFormat:
                try:
                    _ = d.tm_sec
                    t = d  # already in timetuple format
                except AttributeError:
                    t = time.gmtime(d)
                if major:
                    s = time.strftime("%b %d", t)
                else:
                    s = time.strftime("%H%M", t)
        else:
            s = RulerCtrl.LabelString(self, d, major)
        return s

    def step_size(self):
        time_range = self._max - self._min
        seconds_per_pixel = time_range / float(self._length)
        min_pixels_between_ticks = 50
        step = time_steps[min(bisect.bisect(time_steps, abs(seconds_per_pixel * min_pixels_between_ticks)), len(time_steps) - 1)]
        self._minor = step
        self._major = step * 4
        return step, step * 4

    def FindLinearTickSizes(self, UPP):
        UPP = (self._max - self._min)/float(self._length)  # Units per pixel
        print UPP, self._max, self._min, (self._max - self._min), float(self._length)
        self.step_size()

    def Update(self, dc):
        """Recalculate tick marks where days are major and times are minor.

        Unfortunately named in the superclass because Update is a wx.Window
        method to invalidate regions.
        """

        self._maxwidth = self._length
        self._maxheight = 0
        
        step, major_step = self.step_size()

        self._middlepos = []
            
        # Left and Right Edges
        if self._labeledges:
            self.Tick(dc, 0, self._min, True)
            self.Tick(dc, self._length, self._max, True)
        
        # starting point
        value, _ = divmod(self._min - step/2, step)
        value = int(value * step)
        while value < self._max:
            pos = self.value_to_position(value)
            if pos is not None:
                pos = int(math.floor(pos))
                t = time.gmtime(value)
                major = (t.tm_hour == t.tm_min == t.tm_sec == 0)
                print pos, value, t, major
                self.Tick(dc, pos, t, major)
            else:
                print "offscreen", value
            value += step
            
        self._valid = True


class VirtualLabeledRuler(LabeledRuler):
    def __init__(self, parent, id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=wx.STATIC_BORDER, orient=wx.HORIZONTAL):
        self.common_init()

        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)        
        width, height = size

        self._min = 0.0
        self._max = 10.0
        self._orientation = orient
        self._spacing = 5
        self._hassetspacing = False
        self._format = DateFormat
        self._flip = False
        self._log = False
        self._labeledges = False
        self._units = ""
        
        self._drawingparent = None
        self._drawingpen = wx.Pen(wx.BLACK, 2)

        self._left = -1
        self._top = -1
        self._right = -1
        self._bottom = -1

        self._major = 1
        self._minor = 1

        self._indicators = []
        self._currentIndicator = None

        fontsize = 10
        if wx.Platform == "__WXMSW__":
            fontsize = 8

        self._minorfont = wx.Font(fontsize, wx.SWISS, wx.NORMAL, wx.NORMAL)
        self._majorfont = wx.Font(fontsize, wx.SWISS, wx.NORMAL, wx.BOLD)

        if wx.Platform == "__WXMAC__":
            self._minorfont.SetNoAntiAliasing(True)
            self._majorfont.SetNoAntiAliasing(True)

        self._bits = BitSink()
        self._userbits = []
        self._userbitlen = 0
        self._tickmajor = True
        self._tickminor = True
        self._timeformat = IntFormat
        self._labelmajor = True
        self._labelminor = True
        self._tickpen = wx.Pen(wx.BLACK)
        self._textcolour = wx.BLACK
        self._background = wx.WHITE

        self._valid = False
        self._state = 0

        self._style = style
        self._orientation = orient
        wbound, hbound = self.CheckStyle()

        if orient & wx.VERTICAL:
            self.SetBestSize((28, height))
        else:
            self.SetBestSize((width, 28))

        self.SetBounds(0, 0, wbound, hbound)

        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_MOUSE_EVENTS, self.OnMouseEvents)
        self.Bind(wx.EVT_MOUSE_CAPTURE_LOST, lambda evt: True)


open_hand_cursor_data = "eJzrDPBz5+WS4mJgYOD19HAJAtIKIMzBDCRdlnQdA1Is6Y6+jgwMG/u5/ySyMjAwMwT4hLgCxRkZGZmYmJiZmVlYWFhZWdnY2NjZ2Tk4OL58+fLt27fv37//+PHj58+fv379+v37958/f/7+/fvv37////8zjIJRMMSB0ddH84BZgKEkyC/4/8gGDMHf2VWBQSJZ4hpREpyfVlKeWJTKEJCYmVei5+caolBmrGeqZ2Fu3RALVLTV08UxxML/6eRsvmYDnuZYiQ3itUe+22t//uF4WOMW44qQlb72ln6Lkn5uLvBkaN8Uu+A407OX7SsZemyNHO/VftYyUGVUUVoaIlguE8/j80Cm7UWL7OmOnMPNwc9yufM1JjB5XnbL0mi4tjDlk6+YITvrTW0N13xDo+0+Sms/WU4sXikW49iYVtN1MW+a5bnVLSJ/fq9T9XL4fesD88fncZ6TVMqYb8dfM1qbfd4psHTXiRM7nV5zxzyJr2FQZg5cEB8aLgWKeU9XP5d1TglNAKfNkK0="

closed_hand_cursor_data = "eJzrDPBz5+WS4mJgYOD19HAJAtIKIMzBDCRdlnQdA1Is6Y6+jgwMG/u5/ySyMjAwMwT4hLgCxRkZGZmYmJiZmVlYWFhZWdnY2P7//88wCkbBCADl+b/FgFmAoSTIL/j/yAYMwd/ZVYFBIlniGlESnJ9WUp5YlMoQkJiZV6Ln5xqiUGasZ6pnYW7dEAtUlO/p4hhi4f928lmuAwo8zY/Pn/ltv3Gyb2ZX39229q8KHfKZtxReeT9kX8f7Zlt7T1dMcsnHDs8JjTnpnE0cE25w+7i9mHFcYcZ0hlm/LkQwfvPp8s9WbPj2NfiMbIyY8+Gnj/WehlWw7WqJfppXqF//kF3N/9PCPq5ZP2bogeLM09XPZZ1TQhMAielZWg=="

class ZoomRulerBase(object):
    """Base class for zoom ruler, regardless of container.

    self.panel must point to the scrolling window.
    """
    open_hand_cursor_ = None
    closed_hand_cursor_ = None
    zoom_percent = .1  # uses this percentage of control width (in pixels) as zoom distance at each mouse wheel click

    @property
    def can_drag_cursor(self):
        if self.__class__.open_hand_cursor_ is None:
            raw = zlib.decompress(base64.b64decode(open_hand_cursor_data))
            stream = cStringIO.StringIO(raw)
            image = wx.ImageFromStream(stream)
            image.SetOptionInt(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 16)
            image.SetOptionInt(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 16)
            self.__class__.open_hand_cursor_ = wx.CursorFromImage(image)
        return self.__class__.open_hand_cursor_

    @property
    def dragging_cursor(self):
        if self.__class__.closed_hand_cursor_ is None:
            raw = zlib.decompress(base64.b64decode(closed_hand_cursor_data))
            stream = cStringIO.StringIO(raw)
            image = wx.ImageFromStream(stream)
            image.SetOptionInt(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 16)
            image.SetOptionInt(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 16)
            self.__class__.closed_hand_cursor_ = wx.CursorFromImage(image)
        return self.__class__.closed_hand_cursor_

    def populate(self, panel):
        self.panel = panel
        self.ruler = LabeledRuler(self.panel, -1, style=wx.BORDER_NONE)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.ruler, 1, wx.EXPAND, 0)
        self.panel.SetSizer(sizer)
        self.panel.ShowScrollbars(wx.SHOW_SB_NEVER, wx.SHOW_SB_NEVER)
        self.panel.SetScrollRate(1, 0)
        # sizer.Layout()
        self.panel.Fit()

        self.label_max = self.label_min = None

        self.init_events()

    def init_events(self):
        self.panel.Bind(wx.EVT_SCROLLWIN, self.on_scroll)
        self.ruler.Bind(wx.EVT_KEY_DOWN, self.on_key_down)
        self.ruler.Bind(wx.EVT_KEY_UP, self.on_key_up)
        self.ruler.Bind(wx.EVT_MOUSE_EVENTS, self.on_mouse_events)
        self.select_start = None
        self.select_end = None
        self.select_threshold = 2
        self.drag_start = -1
        self.view_at_drag_start = -1
        self.cursor_mode_image = {
            "select": wx.StockCursor(wx.CURSOR_ARROW),
            "drag_mode": self.can_drag_cursor,
            "dragging": self.dragging_cursor,
        }
        self.cursor_mode = "select"

    @property
    def is_drag_mode(self):
        return self.cursor_mode == "drag_mode"

    @property
    def is_dragging(self):
        return self.drag_start >= 0

    @property
    def is_selecting(self):
        return self.select_end is not None

    def set_mode(self, mode):
        if self.cursor_mode != mode:
            self.cursor_mode = mode
            self.SetCursor(self.cursor_mode_image[mode])
            print "mode:", mode

    def on_key_down(self, event):
        if self.is_dragging:
            # mouse up will take care of cursor
            return
        mods = event.GetModifiers()
        if mods == wx.MOD_ALT:
            mode = "drag_mode"
        else:
            mode = "select"
        self.set_mode(mode)

    def on_key_up(self, event):
        if self.is_dragging:
            # mouse up will take care of cursor
            return
        self.set_mode("select")

    def on_mouse_events(self, event):
        """Overriding the ruler to capture wheel events
        """
        wheel_dir = event.GetWheelRotation()
        x, _ = self.panel.GetViewStart()
        pos = event.GetPosition()[0] + x
        mods = event.GetModifiers()
        if wheel_dir:
            if mods == wx.MOD_NONE:
                if wheel_dir < 0:
                    self.zoom_out(pos)
                elif wheel_dir > 0:
                    self.zoom_in(pos)
            event.Skip()

        op = None  # the mouse operation to process
        next_mode = None  # next state after operation
        mode = self.cursor_mode

        # determine state machine command
        if mode == "select":
            if event.ButtonDown():
                if (event.LeftIsDown() and event.AltDown()) or event.MiddleIsDown():
                    op = "start drag"
                elif event.ShiftDown() and self.ruler.has_selection:
                    op = "extend selection"
                else:
                    op = "start selection"
            elif event.LeftUp():
                if self.ruler.has_selection:
                    op = "finish selection"
                elif self.select_start >= 0:
                    op = "select item"
            elif event.Dragging():
                if event.ShiftDown() and self.ruler.has_selection:
                    op = "extend selection"
                elif self.is_selecting:
                    op = "extend selection"
                elif event.LeftIsDown():
                    op = "select threshold"
            elif event.Moving():
                op = "hit test"
            else:
                op = "unknown"

        elif mode == "drag_mode":
            if event.ButtonDown():
                if (event.LeftIsDown() and event.AltDown()) or event.MiddleIsDown():
                    op = "start drag"
            elif event.ButtonUp():
                op = "end drag"
                if event.AltDown():
                    next_mode = "drag_mode"
            elif event.Moving():
                op = "hit test"
                next_mode = "drag_mode"
            else:
                op = "unknown"

        elif mode == "dragging":
            if event.ButtonUp():
                op = "end drag"
                if event.AltDown():
                    next_mode = "drag_mode"
            elif event.Dragging() or (event.LeftIsDown() and event.AltDown()) or event.MiddleIsDown():
                op = "dragging"
            else:
                op = "unknown"

        # print "mouse: state=%s op=%s pos=%d btns=%s%s%s" % (mode, op, pos, "L" if event.LeftIsDown() else "l", "M" if event.MiddleIsDown() else "m", "R" if event.RightIsDown() else "r")

        # process state machine commands
        if op == "start selection":
            self.selection_cleared_callback()
            self.ruler.selected_ranges = []
            self.select_start = self.position_to_value(pos)
            self.select_end = None
            self.Refresh()
        elif op == "select threshold":
            start_pos = self.value_to_position(self.select_start, True)
            if abs(pos - start_pos) > self.select_threshold:
                self.select_end = self.position_to_value(pos)
                self.ruler.selected_ranges = [(self.select_start, self.select_end)]
                self.Refresh()
        elif op == "extend selection":
            last_start, last_end = self.ruler.selected_ranges[-1]
            value = self.position_to_value(pos)
            self.ruler.selected_ranges[-1] = (last_start, value)
            self.Refresh()
        elif op == "finish selection":
            self.selection_finished_callback()
        elif op == "start drag":
            self.drag_start = wx.GetMousePosition()[0]  # absolute mouse pos (see below)
            self.view_at_drag_start, _ = self.panel.GetViewStart()
            next_mode = "dragging"
        elif op == "dragging":
            pos =  wx.GetMousePosition()[0]  # absolute mouse pos
            delta = pos - self.drag_start
            x = self.view_at_drag_start - delta
            self.panel.Scroll(x, 0)
            next_mode = "dragging"
        elif op == "end drag":
            self.drag_start = -1
        elif op == "hit test":
            label = self.ruler.hit_test(pos)
            if label is not None:
                self.over_item_callback(pos, label)
            else:
                self.not_over_item_callback(pos)
        elif op == "select item":
            if self.ruler.has_selection:
                self.ruler.selected_ranges = []
                self.Refresh()
            p = self.value_to_position(self.select_start)
            item = self.ruler.hit_test(p)
            if item is not None:
                self.selected_item_callback(item)
        else:
            print("unknown state for mouse")

        if next_mode is None:
            next_mode = "select"
        self.set_mode(next_mode)
        event.Skip()

    def selection_finished_callback(self):
        print "DONE!"
        items = self.ruler.marks_in_selection()
        print items

    def over_item_callback(self, pos, item):
        print "hit at %d: %s" % (pos, item)

    def not_over_item_callback(self, pos):
        pass

    def selected_item_callback(self, item):
        print "CHOSEN!", item

    def selection_cleared_callback(self):
        print "CLEARED!"

    @property
    def zoom_rate(self):
        return max(10, int(self.zoom_percent * self.ruler.GetSize()[0]))

    def zoom_out(self, pos):
        size = self.ruler.GetSize()
        newsize = size - (self.zoom_rate, 0)
        self.zoom_parent(newsize, pos)

    def zoom_in(self, pos):
        size = self.ruler.GetSize()
        newsize = size + (self.zoom_rate, 0)
        self.zoom_parent(newsize, pos)

    def zoom_parent(self, size, pos):
        """Zoom in or out, maintaining the zoom center at the mouse location
        """
        value = self.ruler.position_to_value(pos)
        pixels_from_left = self.panel.CalcScrolledPosition(pos, 0)[0]

        if size[0] < self.actual_screen_width:
            size[0] = self.actual_screen_width

        self.ruler.SetSize(size)
        print "SIZE", size, type(size)
        self.panel.SetVirtualSize(size)

        new_pos = self.ruler.value_to_position(value)
        new_left = new_pos - pixels_from_left
        self.panel.Scroll(new_left, 0)

        self.update_limits()

    def update_limits(self):
        x, y = self.panel.GetViewStart()
        x1, y1 = self.panel.CalcUnscrolledPosition(x, y)
        left = self.ruler.position_to_value(x1)
        right = self.ruler.position_to_value(x1 + self.GetSize()[0] - 1)
        log.debug("update_limits: view=%d,%d unscrolled=%d,%d parent=%s left=%f right=%f" % (x, y, x1, y1, self.GetSize(), left, right))
        if self.label_min is not None:
            self.label_min.SetLabel(self.ruler.LabelString(left, True))
            self.label_max.SetLabel(self.ruler.LabelString(right, True))

    def on_scroll(self, evt):
        self.Refresh()
        self.update_limits()
        evt.Skip()

    def add_mark(self, timestamp, item):
        self.ruler.set_mark(timestamp, item)

    def rebuild(self, editor):
        log.debug("rebuild")
        timeline_info = editor.get_timeline_info()
        fmt = timeline_info.get("format", "date")
        if fmt == "date":
            self.ruler.SetTimeFormat(DateFormat)
            self.ruler.SetFormat(TimeFormat)
        if fmt == "month":
            self.ruler.SetTimeFormat(MonthFormat)
            self.ruler.SetFormat(TimeFormat)
        else:
            self.ruler.SetFormat(IntFormat)

        self.ruler.clear_marks()
        for start, end, item in timeline_info["marks"]:
            log.debug("adding %s at %s" % (item, start))
            self.add_mark(start, item)
        start = timeline_info["earliest_time"]
        end = timeline_info["latest_time"]
        self.ruler.SetRange(start, end)
        self.update_limits()


class ZoomRulerWithLimits(wx.Panel, ZoomRulerBase):
    """Zoomable ruler that uses a scrollbar and resize to implement the zoom.
    """
    def __init__(self, parent, **kwargs):
        wx.Panel.__init__(self, parent, -1, **kwargs)
        self.panel = scrolled.ScrolledPanel(self, -1, size=(-1,50), style=wx.HSCROLL)
        self.ruler = LabeledRuler(self.panel, -1, style=wx.BORDER_NONE)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.ruler, 1, wx.EXPAND, 0)
        self.panel.SetSizer(sizer)
        self.panel.ShowScrollbars(wx.SHOW_SB_ALWAYS, wx.SHOW_SB_NEVER)
        self.panel.SetupScrolling(scroll_y=False)
        self.panel.SetScrollRate(1, 1)
        # sizer.Layout()
        self.panel.Fit()

        self.label_min = wx.StaticText(self, -1, "Min")
        self.label_max = wx.StaticText(self, -1, "Max", style=wx.ALIGN_RIGHT)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(self.label_min, 0, wx.EXPAND, 0)
        hbox.Add((0, 0), 1)
        hbox.Add(self.label_max, 0, wx.EXPAND, 0)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.panel, 1, wx.EXPAND, 0)
        sizer.Add(hbox, 0, wx.EXPAND, 0)
        self.SetSizerAndFit(sizer)

        self.init_events()


class ZoomRuler(wx.ScrolledWindow, ZoomRulerBase):
    """Zoomable ruler that uses a scrollbar and resize to implement the zoom.
    """
    def __init__(self, parent, **kwargs):
        wx.ScrolledWindow.__init__(self, parent, -1, style=wx.HSCROLL, **kwargs)
        self.populate(self)


class VirtualZoomRuler(wx.ScrolledWindow, ZoomRulerBase, VirtualLabeledRuler):
    """Zoomable ruler that uses a scrollbar and owner-drawing to implement the
    zoom.

    When the zoom is too far in, using a regular window will fail when it gets
    too big. "Too big" is platform dependent, but still it's an unacceptable
    limitation.

    This works around that by handling the scroll offsets ourselves and drawing
    on a virtual canvas.

    This class has multiple parents, and note that we are subclassing from
    RulerCtrl but *not* calling its constructor. We need to override some
    methods, but can't use the constructor because it expects itself to be in a
    real (non-virtual) window.

    The VirtualLabeledRuler constructor takes care of initializing all the
    attributes in RulerCtrl that it needs.
    """
    def __init__(self, parent, **kwargs):
        wx.ScrolledWindow.__init__(self, parent, -1, style=wx.HSCROLL, **kwargs)
        VirtualLabeledRuler.__init__(self, parent)
        self.panel = self
        self.ruler = self

        self.ShowScrollbars(wx.SHOW_SB_NEVER, wx.SHOW_SB_NEVER)
        self.SetScrollbars(1, 0, self._right, 0, 0, 0)
        self.SetScrollRate(1, 0)
        self.SetDoubleBuffered(False)

        self.label_max = self.label_min = None
        self.init_events()

    def get_visible_range(self):
        x, _ = self.GetViewStart()
        return x, x + self.GetSize()[0] - 1

    def SetSize(self, size):
        self.SetBounds(0, 0, size[0], size[1])

    def GetSize(self):
        if self._right == -1:
            size = wx.ScrolledWindow.GetSize(self)
            self._right = 500
            self._bottom = size[1]
        return wx.Size(self._right, self._bottom)

    @property
    def actual_screen_width(self):
        size = wx.ScrolledWindow.GetSize(self)
        return size[0]

    def OnSize(self, event):
        # Need to handle the initial case to get the ruler size up to the
        # window size
        size = event.GetSize()
        if self._right == -1 or self._right < size[0]:
            self._right = size[0]
            self._bottom = size[1]

        width, height = self.CheckStyle()
        self.SetBounds(0, 0, width, height)
        log.debug("new virtual size: %d,%d; as reported %s" % (width, height, self.GetSize()))

        self.Invalidate()
        event.Skip()

    def OnPaint(self, event):
        """
        Handles the ``wx.EVT_PAINT`` event for :class:`RulerCtrl`.

        :param `event`: a :class:`PaintEvent` event to be processed.        
        """

        # dc = wx.PaintDC(self)
        # print dc.GetClippingBox(), "BB", self.IsDoubleBuffered()
        # dc.SetBackground(wx.Brush(self._background))
        # dc.Clear()
        # self.Draw(dc)

        #wx.CallAfter(self.Refresh)
        wx.CallAfter(self.clear_and_paint)

    def clear_and_paint(self):
        dc = wx.ClientDC(self)
        dc.SetBackground(wx.Brush(self._background))
        dc.Clear()
        self.Draw(dc)

    def Update(self):
        wx.ScrolledWindow.Update(self)

ZoomRuler = VirtualZoomRuler


if __name__ == "__main__":
    logging.basicConfig(level=logging.WARNING)

    time_steps = [minute * 60.0 for minute in [1, 2, 5, 10, 15, 20, 30]]
    time_steps.extend([hour * 3600.0 for hour in [1, 2, 4, 6, 12]])
    time_steps.extend([day * 86400.0 for day in [1, 2, 3, 7, 14, 28]])
    visible_pixels = 800
    time_range = 100000  # seconds
    seconds_per_pixel = 100000 / 800
    step = time_steps[min(bisect.bisect(time_steps, abs(seconds_per_pixel)), len(time_steps) - 1)]
    print time_steps
    print step

    b = BitSink()
    print b[1000000]

    class SampleData(object):
        @classmethod
        def get_timeline_info(cls):
            if True:
                fmt = "month"
                start = time.time()
                end = start + 86400 * 10
            else:
                fmt = "int"
                start = 0
                end = 1000

            info = {
                "format": fmt,
                "earliest_time": start,
                "latest_time": end,
                "marks": [(random.uniform(start, end), 0.0, "item % d" % i) for i in range(20)],
            }
            return info


    app = wx.PySimpleApp()
    frm = wx.Frame(None,-1,"Test",style=wx.TAB_TRAVERSAL|wx.DEFAULT_FRAME_STYLE,
                   size=(800,400))
    panel = wx.Panel(frm)
    sizer = wx.BoxSizer(wx.VERTICAL)

    text = wx.StaticText(panel, -1, "Just a placeholder here.")
    sizer.Add(text, 1, wx.EXPAND)

    scroll = VirtualZoomRuler(panel)
    sizer.Add(scroll, 0, wx.EXPAND)
    scroll.rebuild(SampleData)

    panel.SetAutoLayout(True)
    panel.SetSizer(sizer)
    #sizer.Fit(panel)
    #sizer.SetSizeHints(panel)
    panel.Layout()
    app.SetTopWindow(frm)
    frm.Show()

    app.MainLoop()



# Standard library imports.
import os

import wx

# Enthought library imports.
from pyface.tasks.api import DockPane
from traits.api import Event, File, List, Str


class FileBrowserPane(DockPane):
    """ A simple file browser pane.
    """

    #### TaskPane interface ###################################################

    id = 'omnivore.framework.file_browser_pane'
    name = 'File Browser'

    #### FileBrowserPane interface ############################################

    # The list of wildcard filters for filenames.
    filters = List(Str)

    def create_contents(self, parent):
        control = wx.GenericDirCtrl(parent, -1, size=(200,-1), style=wx.NO_BORDER)
        control.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.on_selected)
        return control

    def on_selected(self, evt):
        selected_file = self.control.GetFilePath()
        if selected_file:
            wx.CallAfter(self.task.window.application.load_file, selected_file, self.task, in_current_window=True)
        else:
            # blank file path means that a directory was double-clicked instead
            # of a file.  Skipping the event will expand the directory.
            evt.Skip()


class PythonScriptBrowserPane(FileBrowserPane):
    """ A file browser pane restricted to Python scripts.
    """

    #### TaskPane interface ###################################################

    id = 'omnivore.framework.python_script_browser_pane'
    name = 'Script Browser'

    #### FileBrowserPane interface ############################################

    filters = [ '*.py' ]



#!/usr/bin/env python

import os, sys

class MossPolygon():
    def __init__(self):
 
        self.outerBoundary = [] # a coordinate list
        self.innerBoundaries = [] # a list of coordinate lists
        
    def Print(self):
        print "outerBoundary:",self.outerBoundary
        print "innerBoundaries:",self.innerBoundaries



def ReadMossPolygons(mossBaseFileName, printDiagnostic = False):
    '''
    .ms1 file is fixed format
    
    Header lines 56 characters
    char 1-5 : Item Number(NEGATIVE IF THE COORDINATES ARE LON/LAT)
    char 16-45: Attribute Name
    char 51-55: Number of coord. pairs
    
    X,Y, Coordinate pairs 23 characters
    01-11: x coordinate
    12-22: y coordinate
    
    Long,Lat pairs
    char 01-10: LONGITUDE
    char 11-20: LATITUDE
    char 21-22: FLAG 
        0-NORMAL 
        1-INDICATES FIRST POINT OF ISLAND POLYGON
    
    '''
    # import shapely here so it won't be imported if not needed
    from shapely.geometry import Polygon, MultiPolygon

    # what about MAPBOUND, EXTENDEDOUTLOOKTHREAT
    attributesToRead = ["MAPLAND","FORECASTHEAVY","FORECASTMEDIUM","FORECASTLIGHT","FORECASTUNCERTAINTY"]
    
    landBoundariesList = [] 
    heavyBoundariesList = [] 
    mediumBoundariesList = [] 
    lightBoundariesList = []
    uncertaintyBoundariesList = []
     
    extension = ".ms1"
    if os.path.exists(mossBaseFileName + extension):
        inFile = file(mossBaseFileName + extension, 'rU')
        alreadyReadNextLine = False
        while True:  # read the header lines     
            if not alreadyReadNextLine:
                line = inFile.readline()
            alreadyReadNextLine = False
            
            if not line: break # end of this file
            if line.strip() == "" : continue; # blank line
            
            itemNum = int(line[0:5])
            assert itemNum < 0 # we expect long/lat values, so the itemNum should be negative
            
            attributeName = line[15:45].strip()

            numCoordinates = int(line[50:55])
            
            
            # note: for some versions of GNOME analyst
            # the number of coordinates in MAPLAND overflows and is reported as a negative number or incorrect number.
            # To support those files, we will ignore the number and just read lines based on the length of the lines
              
            if printDiagnostic: 
                print itemNum,attributeName,numCoordinates
            
            readingOuterBoundary = True
            coordList = []
            
            outerBoundary = None
            innerBoundaries = []
             
            if numCoordinates <= 0:
                print "*** ignoring bad %s header line value: numCoordinates: %d ***"%(attributeName,numCoordinates)
            
            # read the points for the polygon for this header           
            while True: #for i in range(0,numCoordinates):
                # since we don't want to rely on numCoordinates
                # we need to look to see if this is the end of this block of coordinates
                ##################
                line = inFile.readline()
                
                # The lines are fixed format,
                # read until we find a header line
                # header lines lines are longer than coordinate lines 
                if (not line) or len(line.strip()) > 50 : # must be end of file or a header line
                    alreadyReadNextLine = True 
                    break # out of this while loop
                
                if attributeName in attributesToRead:
                    #process this line
                    longitudeStr = line[0:10].strip()
                    latitudeStr = line[10:20].strip()
                    flag = line[20:22].strip()
                    
                    if flag == "1" :
                        #then we are starting a new "inner hole"
                        
                        # enforce having the last point of the polygon equal the first point
                        if len(coordList) > 0 and coordList[0] != coordList[-1]:
                            coordList.append(coordList[0])

                        # save the previous coordList
                        if len(coordList) >= 4: # less than 4 would be a degenerate case                          
                            if readingOuterBoundary:
                                outerBoundary = coordList
                            else:
                                innerBoundaries.append(coordList)
                                
                        # reset the coordinate list 
                        coordList = []                                         
                        readingOuterBoundary = False
                    
                    coordList.append((float(longitudeStr),float(latitudeStr)))
                ############# end of while loop
                
            # finished reading the header
            # record the lists we have filled in
            
            if not attributeName in attributesToRead:
                continue # on to the next header line 
            
            # enforce having the last point of the polygon equal the first point
            if len(coordList) > 0 and coordList[0] != coordList[-1]:
                coordList.append(coordList[0])
                
            #filter out degenerate cases
            if len(coordList) < 4: # less than 4 would be a degenerate case
                print "*** ignoring degenerate polygon ***"
                continue # on to the next header line 
                
            # save the coordinate list                             
            if readingOuterBoundary:
                outerBoundary = coordList
            else:
                innerBoundaries.append(coordList)
                
            
            #save thisPolygon
            if len (outerBoundary) > 0 :
                # outerBoundary,innerBoundaries = VerifyAndFixGnomeAnalystPolygon(attributeName,outerBoundary,innerBoundaries)
                if len (outerBoundary) > 0 :
                    thisPolygon = (outerBoundary,innerBoundaries)
                    if attributeName == "MAPLAND": landBoundariesList.append(thisPolygon)
                    elif attributeName == "FORECASTHEAVY": heavyBoundariesList.append(thisPolygon)
                    elif attributeName == "FORECASTMEDIUM": mediumBoundariesList.append(thisPolygon)
                    elif attributeName == "FORECASTLIGHT": lightBoundariesList.append(thisPolygon)
                    elif attributeName == "FORECASTUNCERTAINTY": uncertaintyBoundariesList.append(thisPolygon) 
                
                
        inFile.close()
        
    # convert the lists of MossPolygons to shapely MultiPolygons
    if len(landBoundariesList) == 0: landPolygons = None
    else: 
        landPolygons = MultiPolygon(landBoundariesList)
        if not landPolygons.is_valid:
            # try analysing and fixing the problem
            landPolygons = DiagnoseAndFixMultiPolygon("MAPLAND",landBoundariesList)
    
    if len(heavyBoundariesList) == 0: heavyPolygons = None
    else: 
        heavyPolygons = MultiPolygon(heavyBoundariesList)
        if not heavyPolygons.is_valid:
            heavyPolygons =DiagnoseAndFixMultiPolygon("FORECASTHEAVY",heavyBoundariesList)
    
    if len(mediumBoundariesList) == 0: mediumPolygons = None
    else: 
        mediumPolygons = MultiPolygon(mediumBoundariesList)
        if not mediumPolygons.is_valid:
            mediumPolygons = DiagnoseAndFixMultiPolygon("FORECASTMEDIUM",mediumBoundariesList)
    
    if len(lightBoundariesList) == 0: lightPolygons = None
    else: 
        lightPolygons = MultiPolygon(lightBoundariesList)
        if not lightPolygons.is_valid:
            lightPolygons = DiagnoseAndFixMultiPolygon("FORECASTLIGHT",lightBoundariesList)
                              
    if len(uncertaintyBoundariesList) == 0: uncertaintyPolygons = None
    else: 
        uncertaintyPolygons = MultiPolygon(uncertaintyBoundariesList)
        if not uncertaintyPolygons.is_valid:
            uncertaintyPolygons = DiagnoseAndFixMultiPolygon("FORECASTUNCERTAINTY",uncertaintyBoundariesList)
        
   
    # clip the oil contours to the shoreline
    # note: we need to check that the polygons are valid before trying to clip to prevent shapely from crashing
    if landPolygons != None :
        if landPolygons.is_valid == False:
            print "*** landPolygons is not valid. We will not clip to the shoreline. ***"
        else :
            if heavyPolygons != None: 
                if heavyPolygons.is_valid == False:
                    print "*** heavyPolygons is not valid. It will not be clipped to the shoreline. ***"
                elif heavyPolygons.intersects(landPolygons):
                    print "clipping heavyPolygons to shoreline"
                    heavyPolygons = heavyPolygons.difference(landPolygons)
                    
            if mediumPolygons != None: 
                if mediumPolygons.is_valid == False:
                    print "*** mediumPolygons is not valid. It will not be clipped to the shoreline. ***"
                elif mediumPolygons.intersects(landPolygons):
                    print "clipping mediumPolygons to shoreline"
                    mediumPolygons = mediumPolygons.difference(landPolygons)
                    
            if lightPolygons != None: 
                if lightPolygons.is_valid == False:
                    print "*** lightPolygons is not valid. It will not be clipped to the shoreline. ***"
                elif lightPolygons.intersects(landPolygons):
                    print "clipping lightPolygons to shoreline"
                    lightPolygons = lightPolygons.difference(landPolygons)
                    
            # note: JerryM wonders we should clip the uncertainty to the shoreline.  That it looks better as simple polygons going over the land.
            if uncertaintyPolygons != None: 
                if uncertaintyPolygons.is_valid == False:
                    print "*** uncertaintyPolygons is not valid. It will not be clipped to the shoreline or oil polygons ***"
                else:
                    print "clipping uncertaintyPolygons to shoreline and oil polygons"
                    
                    for polygons,nameOfPolygons in [(lightPolygons,"lightPolygons"),(mediumPolygons,"mediumPolygons"),(heavyPolygons,"heavyPolygons"),(landPolygons,"landPolygons")]:
                        if polygons != None: 
                            print "taking difference with",nameOfPolygons
                            newUncertaintyPolygons = uncertaintyPolygons.difference(polygons)
                            print "finished taking difference"
                            if not newUncertaintyPolygons.is_valid:
                                #print "Uncertainty Polygon is no longer valid after taking difference with",polygons,nameOfPolygons
                                s = "*** uncertaintyPolygons have not been clipped to %s ***"%(nameOfPolygons)
                                print s
                            else:
                                uncertaintyPolygons = newUncertaintyPolygons
                                
                    
   
           
    return   (landPolygons,heavyPolygons,mediumPolygons,lightPolygons,uncertaintyPolygons)


def VerifyAndFixGnomeAnalystPolygon(attributeName,outerBoundary,innerBoundaries):
    '''
    GNOME Analyst sometimes generated invalid polygons.
    
    One case was with a MAPLAND polygon and seemed to be caused by a clipping routine (perhaps in DOGS) where the outer boundary crossed itself.
    There is nothing we can automatically due for suce a case, so we omit such polygons and inform the user.
    
    Another case had a spurious hole inside another hole n contours output by GNOME Analyst.
    . Rather than figure it out in GNOME Analyst,
    A will instead just leave out any bad holes. If we miss having a hole in the polygon, it is probably not a big deal.
    '''
    # first see if the polygon is valid
    shapelyPolygon = Polygon(outerBoundary,innerBoundaries)
    if shapelyPolygon.is_valid:
        return (outerBoundary,innerBoundaries) # this is a good polygon
    # alright, check to see if the outerBoundary is valid
    shapelyPolygon = Polygon(outerBoundary)
    if not shapelyPolygon.is_valid:
        # the problem is one we can't fix
        print "*** ERROR: bad %s outerBoundary ***"%(attributeName)
        print "Can't fix this problem, so omitting this polygon."
        if attributeName ==  "MAPLAND": print "** Minor error: The result is that the oil will not be clipped to this island."
        else: print "*** IMPORTANT ERROR:  an oiled area is being omitted."
        print outerBoundary
        return ([],[]) # can't fix it , so return empty lists, so that this part gets skipped
    # if we get here the outer boundary is OK
    # try eliminating any bad holes
    goodHoles = []
    innerBoundariesToTry = []
    for hole in innerBoundaries:
        shapelyPolygon = Polygon(outerBoundary,goodHoles + [hole])
        if shapelyPolygon.is_valid:
            goodHoles.append(hole)
        else:
            print "*** omitting bad hole in %s ***"%(attributeName)
            print hole
                  
    return (outerBoundary,goodHoles)
 
 
def DiagnoseAndFixMultiPolygon(attributeName,boundariesList, printDiagnostic = False):
    print "---"
    print "*** %s is invalid. Running diagnostic... ***"%(attributeName)
    # Note: we have already tested each polygon, so the problem is when we add them to a multipolygon
    # the likely cause is overlapping polygons, so we will automatically union those polygons 
    #printDiagnostic = True
    goodPolygons = []
    badPolygons = []
    useSlowerCode = False
    numPolygons = len(boundariesList)
    if useSlowerCode:
        for i in range(0,numPolygons) :
            if i % 20 == 0:
                print "processing polygon", i
            thisPolygon = boundariesList[i]
            shapelyMultiPoly = MultiPolygon(goodPolygons + [thisPolygon])
            if shapelyMultiPoly.is_valid:
                goodPolygons.append(thisPolygon)
            else:
                print "*** problem when adding polygon %d ***"%(i)
                print thisPolygon
                badPolygons.append(thisPolygon)
    else:
        ''' it can be very slow trying these one at a time, 
        so lets try jumping when we can and dropping back to one at a time when we have a problem
        '''
        numPolygons = len(boundariesList)
        if printDiagnostic: print "numPolygons",numPolygons
        
        i = 0
        lineReported = 0
        while True:
            if i >= numPolygons:
                if printDiagnostic: print "reached end of list"
                break # end of list
            
            lineToReport = (i/100)*100
            if lineToReport != lineReported:
                lineReported = lineToReport
                if i > 0:
                    print "processing polygon %d of %d"%(i,numPolygons)
                
            continueToTop = False
            for numToJump in [100,50,20,10]:
                if continueToTop: continue
                if i + numToJump > numPolygons:
                    numToJump = numPolygons - i;
                    if printDiagnostic: print "adjusting numToJump to",numToJump
                #try the jump
                if printDiagnostic: print "trying jump"
                polygonsToAdd = boundariesList[i:i+numToJump]
                shapelyMultiPoly = MultiPolygon(goodPolygons + polygonsToAdd)
                if shapelyMultiPoly.is_valid:
                    goodPolygons = goodPolygons + polygonsToAdd
                    i = i + numToJump
                    if printDiagnostic: print "jumped to %i"%(i)
                    continueToTop = True
                
            if continueToTop: continue
                            
            #resort to one at a time
            if printDiagnostic: print "resorting to one at a time. i = %d"%(i)
            for j in range(0,numToJump):              
                thisPolygon = boundariesList[i]
                shapelyMultiPoly = MultiPolygon(goodPolygons + [thisPolygon])
                if shapelyMultiPoly.is_valid:
                    goodPolygons.append(thisPolygon)
                else:
                    if True: print "polygon %d is bad"%(i)
                    badPolygons.append(thisPolygon)
                i = i+1

    # now automatically generate a "fixed" multipolygon by unioning the bad polygons
    shapelyMultiPoly = MultiPolygon(goodPolygons)
    if len(badPolygons) > 0: 
        print "---"
        print "*** Handling bad polygons ***"
        for poly in badPolygons:
            print "Bad polygon:", poly
            islandPoly = MultiPolygon([poly])
            if islandPoly.is_valid:
                print "Fixed: The bad polygon was a valid polygon, it has been unioned to the whole."
                shapelyMultiPoly = shapelyMultiPoly.union(islandPoly)
            else:
                # try to fix this polygon
                '''
                Outer boundary problems:
                Sometimes with clipping of shoreline, there are cases of the outer boundary of the
                shoreline crossing itself.  There is not really anything we can do about such cases.
                '''
                # check to see if the outer boundary is valid
                outerBoundary,innerBoundaries = poly
                
                islandPoly = MultiPolygon([(outerBoundary,[])])
                if not islandPoly.is_valid:
                    print "Outer boundary is not valid."
                    print "Unable to fix this polygon."
                    if attributeName == "MAPLAND":
                        print "This is a minor error, it just means the oil contours will not be clipped to this part of the land." 
                    else:
                        print "This is a serious error.  Part of the %s area will be missing."%(attributeName)
                    print "---"
                    continue # we will omit this polygon
                
                print "The outer boundary of the polygon is valid."
                
                
                ##############################
                '''
                Hole problems:
                There are two kinds of problems that can occur with GNOME Analyst contours.
                
                Sometimes the holes stick slightly out of the outerboundary.
                In such a case we will rely on the fact that the holes were just areas to be removed.
                
                Sometimes there seem to be holes within holes. 
                I'm not sure why GNOME Analyst is doing that, but we will assume that 
                the holes were just areas to be removed.
                '''
                numHoles = len(innerBoundaries)
                if numHoles > 0: print "Examing the %d holes..."%(numHoles)
                assert numHoles > 0 # the only way to get to this part of the code is for a hole to be causing the problem
                numOmittedHoles = 0
                for innerBoundary in innerBoundaries:
                    hole = Polygon(innerBoundary)
                    if not hole.is_valid:
                        numOmittedHoles = numOmittedHoles + 1
                        print "Hole is not valid:", hole 
                        print "Omitting this hole. This is a minor error."
                    else:
                        # subtract this hole from the islandPolygon
                        islandPoly = islandPoly.difference(hole)
                    
                if numOmittedHoles > 0: 
                    print "Partially fixed: %d invalid holes were not subtracted from this polygon, but this polygon has been unioned to the whole."%(numOmittedHoles)
                elif numHoles > 0: 
                    print "Fixed: all holes successfully subtracted from this polygon and the polygon unioned to the whole."
                else : 
                    print "Fixed: polygon unioned to the whole."
                
                shapelyMultiPoly = shapelyMultiPoly.union(islandPoly)
                
            print "---"
        
    return shapelyMultiPoly
     
        
def ReadSpillInfo(mossBaseFileName):
    '''
    0, SPILLID: 
    0, FROM: 
    0, CONTACT: 
    0, ISSUED: 13:20, 7/24/09
    0, VALIDFOR: 06:00, 7/17/09
    0, ADDLEDATA: 
    0, CAVEAT: This trajectory was produced by GNOME (General NOAA Operational Modeling Environment),
    0, CAVEAT: and should be used for educational and planning purposes only--not for a real response. In the
    0, CAVEAT: event of an oil or chemical spill in U.S. waters, contact the U.S. Coast Guard National Response
    0, CAVEAT: Center at 1-800-424-8802.
    0, CAVEAT: 
    0, FROMLOGO: gnome.bmp
    0, LIGHTHEAVYRANGE: 3,5    
    '''
    spillInfo = {} 
    extension = ".ms3"
    if os.path.exists(mossBaseFileName + extension):
        "reading spill info from:", mossBaseFileName + extension
        inFile = open(mossBaseFileName + extension, 'rU')
        while True:       
            line = inFile.readline()
            if not line: break # end of this file
            #print line
            if line.strip() == "" : continue; # blank line
            if line[0:2] != "0," :# lines that start with "0," are the header lines
                break; # we are past the header lines, we don't need to read anymore of this file
            else:
                line = line[2:].strip()
                key = line.split()[0]
                value = line[len(key):].strip()
                # remove the : char from the keyword (should be the last char
                assert key[-1] == ':'
                key = key[:-1] # remove the last char
                if key in spillInfo: 
                    if spillInfo[key] == "":
                        spillInfo[key] = value
                    else: # concatenate the strings
                        spillInfo[key] = spillInfo[key] + " " + value
                else:
                    spillInfo[key] = value
                    
                #print key,value
                
        inFile.close()
        
        # CAVEAT and FROMLOGO were not in the original NOAA standard, so add them if they are not there
        for key in ["CAVEAT","FROMLOGO","LIGHTHEAVYRANGE"]:
            spillInfo.setdefault(key, "") # setdefault will set the key to the default only if it is not already set
                
    else:
        print "File does not exist:", mossBaseFileName + extension
    return spillInfo

    
'''
0, SPILLID: 
0, FROM: 
0, CONTACT: 
0, ISSUED: 13:20, 7/24/09
0, VALIDFOR: 06:00, 7/17/09
0, ADDLEDATA: 
0, CAVEAT: This trajectory was produced by GNOME (General NOAA Operational Modeling Environment),
0, CAVEAT: and should be used for educational and planning purposes only--not for a real response. In the
0, CAVEAT: event of an oil or chemical spill in U.S. waters, contact the U.S. Coast Guard National Response
0, CAVEAT: Center at 1-800-424-8802.
0, CAVEAT: 
0, FROMLOGO: noaa.bmp
'''

class LE():
    def __init__(self):
 
        self.itemNum = None
        self.latitudeStr = None
        self.longitudeStr = None
        #
        self.type = None
        self.pollutant = None
        self.depthInMetersStr = None
        self.massInKilogramsStr = None
        self.densityInGramsPerCubicCentimeterStr = None
        self.ageInHoursSinceReleaseStr = None # tech doc said age in seconds, but it has always been age in hours
        self.status = None
        
    def Print(self):
        print self.itemNum,self.latitudeStr,self.longitudeStr,self.type,self.status
    def __str__(self): 
        return str( (self.itemNum, self.latitudeStr, self.longitudeStr, self.type,self.status), )


def ReadMossLeFiles(mossBaseFileName):
    """
    Reads an LE moss file, and returns a pair of  lists of LE objects , (blackLEs,redLEs)     
    """

    blackLEs = []
    redLEs = []
    leGroups = [(blackLEs,".ms4",".ms5"),(redLEs,".ms6",".ms7")]
            
    for LEs,coordinateExtension,propertyExtension in leGroups:
        if not os.path.exists(mossBaseFileName + coordinateExtension):
            continue # only read the files if we have the coordinates file
        
        coordinatesFile = file(mossBaseFileName + coordinateExtension, 'rU')
        
        if(os.path.exists(mossBaseFileName + propertyExtension)):
            propertiesFile = file(mossBaseFileName + propertyExtension, 'rU')
        else :
            propertiesFile = None
            
        while True:       
            # read the LE coordinates
            # first line holds the item number (negative) 
            line = coordinatesFile.readline()
            if not line: break # end of this file
            if line.strip() == "" : break; # blank line, end of this file
            assert line[15:23] == "LE POINT" 
            thisLE = LE()        
            thisLE.itemNum = int(line[0:5])
            # the next line holds the coordinates
            line = coordinatesFile.readline()
            thisLE.longitudeStr,thisLE.latitudeStr = line.split()[:2]
             
            # read the LE properties
            if(propertiesFile):
                # Note: we will rely on the LEs being in the same order, but we verify that
                line = propertiesFile.readline()
                properties = line.split(",")
                #print properties
                i = 0
                      
                assert thisLE.itemNum == int(properties[i]) # verify the LEs (itemNums) are in the same order
                i = i + 1
                 
                thisLE.type,i = properties[i].strip(),i+1
                assert thisLE.type in ["ABSOLUTEMASS", "RELATIVEMASS", "ABSOLUTEPROBABILITY", "RELATIVEPROBABILITY"]
               
                thisLE.pollutant,i = properties[i].strip(),i+1
                assert thisLE.pollutant in ["GAS", "JP4", "JP5", "DIESEL",
                    "IFO", "BUNKER", "LIGHTCRUDE", "MEDIUMCRUDE", "HEAVYCRUDE", "LAPIO",
                    "CONSERVATIVE"]
                
                thisLE.depthInMetersStr,i = properties[i].strip(),i+1
                thisLE.massInKilogramsStr,i = properties[i].strip(),i+1
                thisLE.densityInGramsPerCubicCentimeterStr,i = properties[i].strip(),i+1
                thisLE.ageInHoursSinceReleaseStr,i = properties[i].strip(),i+1
                thisLE.status,i = properties[i].strip(),i+1
                assert thisLE.status in ["INWATER", "ONBEACH", "OFFMAP"]
                             
            LEs.append(thisLE)
            #thisLE.Print()
                     
        coordinatesFile.close()
        if propertiesFile != None:
            propertiesFile.close()
    
    return (blackLEs,redLEs)


def ReadMossFiles(inputPath):
    # read each of the files that exist
    # verify that we have a file of the right extension
    if not os.path.exists(inputPath):
        print "File does not exist:",inputPath 
        return
    allowedExtentions = [".MS1",".MS2",".MS3",".MS4",".MS5",".MS6",".MS7"]
    extention = inputPath[-4:].upper() #
    if not extention.upper() in allowedExtentions:
        print "File name must end in" + allowedExtentions
        return
    
 
def FilterLEsByStatus(LEs, statusVal):
    matchingLEs = []
    
    for le in LEs:
        if le.status == statusVal:
            matchingLEs.append(le)
              
    if len(matchingLEs) == 0:
        matchingLEs = None
    return matchingLEs

#############################
#############################           
if __name__ == "__main__":
    import sys
    if len(sys.argv) > 1:
        filePath = sys.argv[1]
        ext = filePath.split('.')[-1]
        if len(ext) == 3 and ext[:2].lower() == 'ms':
            # this looks like it has a moss extension, so strip it.
            filePath = filePath[:-4]
            
        printDiagnostic = True
        ReadMossPolygons(filePath,printDiagnostic)
    else:
        ''' 
        If called with no argument, run the sample files.
        '''
        print "***************"
        print "No argument provided.  Processing the sample files..."
        print "***************"
        os.chdir(os.path.dirname(sys.argv[0]))

        inputBasePath = "samplemossfiles/test1/tests"
        #spillInfo = ReadSpillInfo(inputBasePath)
        #print spillInfo
        #blackLEs,redLEs = ReadMossLeFiles(inputBasePath)
        printDiagnostic = False
        
        #inputBasePath = "samplemossfiles/mcity/Mcity.14.michael"
        inputBasePath = "samplemossfiles/boston test/boston test"
        inputBasePath = "samplemossfiles/bad map/3-23-0600"
    
        landPolygons,heavyPolygons,mediumPolygons,lightPolygons,uncertaintyPolygons = ReadMossPolygons(inputBasePath,printDiagnostic)
        #print heavyPolygons,mediumPolygons,lightPolygons
        print landPolygons
        print heavyPolygons
        print mediumPolygons
        print lightPolygons
     
    print "done"


#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/07/2004
#
#------------------------------------------------------------------------------

""" Defines the Handler class used to manage and control the editing process in
    a Traits-based user interface.
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------

from __future__ import absolute_import

from .toolkit import toolkit

from .help import on_help_call

from .view_element import ViewElement

from .helper import user_name_for

from .ui_info import UIInfo

from traits.api import HasPrivateTraits, HasTraits, Instance

#-------------------------------------------------------------------------
#  Closes a DockControl (if allowed by the associated traits UI Handler):
#-------------------------------------------------------------------------


def close_dock_control(dock_control):
    """ Closes a DockControl (if allowed by the associated Traits UI Handler).
    """
    # Retrieve the traits UI object set when we created the DockControl:
    ui = dock_control.data

    # Ask the traits UI handler if it is OK to close the window:
    if not ui.handler.close(ui.info, True):
        # If not, tell the DockWindow not to close it:
        return False

    # Otherwise, clean up and close the traits UI:
    ui.dispose()

    # And tell the DockWindow to remove the DockControl:
    return True

#-------------------------------------------------------------------------
#  'Handler' class:
#-------------------------------------------------------------------------


class Handler(HasPrivateTraits):
    """ Provides access to and control over the run-time workings of a
    Traits-based user interface.
    """

    #-------------------------------------------------------------------------
    #  Informs the handler what the UIInfo object for a View will be:
    #-------------------------------------------------------------------------

    def init_info(self, info):
        """ Informs the handler what the UIInfo object for a View will be.

            This method is called before the UI for the View has been
            constructed. It is provided so that the handler can save the
            reference to the UIInfo object in case it exposes viewable traits
            whose values are properties that depend upon items in the context
            being edited.
        """
        pass

    #-------------------------------------------------------------------------
    #  Initializes the controls of a user interface:
    #-------------------------------------------------------------------------

    def init(self, info):
        """ Initializes the controls of a user interface.

        Parameters
        ----------
        info : UIInfo object
            The UIInfo object associated with the view

        Returns
        -------
        A Boolean, indicating whether the user interface was successfully
        initialized. A True value indicates that the UI can be displayed;
        a False value indicates that the display operation should be
        cancelled. The default implementation returns True without taking
        any other action.

        Description
        -----------
        This method is called after all user interface elements have been
        created, but before the user interface is displayed. Override this
        method to customize the user interface before it is displayed.
        """
        return True

    #-------------------------------------------------------------------------
    #  Positions a dialog-based user interface on the display:
    #-------------------------------------------------------------------------

    def position(self, info):
        """ Positions a dialog-based user interface on the display.

        Parameters
        ----------
        info : UIInfo object
            The UIInfo object associated with the window

        Returns
        -------
        Nothing.

        Description
        -----------
        This method is called after the user interface is initialized (by
        calling init()), but before the user interface is displayed. Override
        this method to position the window on the display device. The default
        implementation calls the position() method of the current toolkit.

        Usually, you do not need to override this method, because you can
        control the window's placement using the **x** and **y** attributes
        of the View object.
        """
        toolkit().position(info.ui)

    #-------------------------------------------------------------------------
    #  Handles a request to close a dialog-based user interface by the user:
    #-------------------------------------------------------------------------

    def close(self, info, is_ok):
        """ Handles the user attempting to close a dialog-based user interface.

        Parameters
        ----------
        info : UIInfo object
            The UIInfo object associated with the view
        is_ok : Boolean
            Indicates whether the user confirmed the changes (such as by
            clicking **OK**.)

        Returns
        -------
        A Boolean, indicating whether the window should be allowed to close.

        Description
        -----------
        This method is called when the user attempts to close a window, by
        clicking an **OK** or **Cancel** button, or clicking a Close control
        on the window). It is called before the window is actually destroyed.
        Override this method to perform any checks before closing a window.

        While Traits UI handles "OK" and "Cancel" events automatically, you
        can use the value of the *is_ok* parameter to implement additional
        behavior.
        """
        return True

    #-------------------------------------------------------------------------
    #  Handles a dialog-based user interface being closed by the user:
    #-------------------------------------------------------------------------

    def closed(self, info, is_ok):
        """ Handles a dialog-based user interface being closed by the user.

        Parameters
        ----------
        info : UIInfo object
            The UIInfo object associated with the view
        is_ok : Boolean
            Indicates whether the user confirmed the changes (such as by
            clicking **OK**.)

        Description
        -----------
        This method is called *after* the window is destroyed. Override this
        method to perform any clean-up tasks needed by the application.
        """
        return

    #-------------------------------------------------------------------------
    #  Handles the 'Revert' button being clicked:
    #-------------------------------------------------------------------------

    def revert(self, info):
        """ Handles the **Revert** button being clicked.
        """
        return

    #-------------------------------------------------------------------------
    #  Handles the 'Apply' button being clicked:
    #-------------------------------------------------------------------------

    def apply(self, info):
        """ Handles the **Apply** button being clicked.
        """
        return

    #-------------------------------------------------------------------------
    #  Shows the help associated with the view:
    #-------------------------------------------------------------------------

    def show_help(self, info, control=None):
        """ Shows the help associated with the view.

        Parameters
        ----------
        info : UIInfo object
            The UIInfo object associated with the view
        control : UI control
            The control that invokes the help dialog box

        Description
        -----------
        This method is called when the user clicks a **Help** button in a
        Traits user interface. The method calls the global help handler, which
        might be the default help handler, or might be a custom help handler.
        See **traitsui.help** for details about the setting the
        global help handler.
        """
        if control is None:
            control = info.ui.control
        on_help_call()(info, control)

    #-------------------------------------------------------------------------
    #  Handles setting a specified object trait's value:
    #-------------------------------------------------------------------------

    def setattr(self, info, object, name, value):
        """ Handles the user setting a specified object trait's value.

        Parameters
        ----------
        object : object
            The object whose attribute is being set
        name : string
            The name of the attribute being set
        value
            The value to which the attribute is being set

        Description
        -----------
        This method is called when an editor attempts to set a new value for
        a specified object trait attribute. Use this method to control what
        happens when a trait editor tries to set an attribute value. For
        example, you can use this method to record a history of changes, in
        order to implement an "undo" mechanism. No result is returned. The
        default implementation simply calls the built-in setattr() function.
        If you override this method, make sure that it actually sets the
        attribute, either by calling the parent method or by setting the
        attribute directly

        """
        setattr(object, name, value)

    #-------------------------------------------------------------------------
    #  Gets a specified View object:
    #-------------------------------------------------------------------------

    def trait_view_for(self, info, view, object, object_name, trait_name):
        """ Gets a specified View object.
        """
        # If a view element was passed instead of a name or None, return it:
        if isinstance(view, ViewElement):
            return view

        # Generate a series of possible view or method names of the form:
        # - 'view'
        #   trait_view_for_'view'( object )
        # - 'class_view'
        #   trait_view_for_'class_view'( object )
        # - 'object_name_view'
        #   trait_view_for_'object_name_view'( object )
        # - 'object_name_class_view'
        #   trait_view_for_'object_name_class_view'( object )
        # where 'class' is the class name of 'object', 'object' is the object
        #       name, and 'name' is the trait name. It returns the first view
        #       or method result which is defined on the handler:
        klass = object.__class__.__name__
        cname = '%s_%s' % (object_name, trait_name)
        aview = ''
        if view:
            aview = '_' + view
        names = ['%s_%s%s' % (cname, klass, aview),
                 '%s%s' % (cname, aview),
                 '%s%s' % (klass, aview)]
        if view:
            names.append(view)
        for name in names:
            result = self.trait_view(name)
            if result is not None:
                return result
            method = getattr(self, 'trait_view_for_%s' % name, None)
            if callable(method):
                result = method(info, object)
                if result is not None:
                    return result

        # If nothing is defined on the handler, return either the requested
        # view on the object itself, or the object's default view:
        return object.trait_view(view) or object.trait_view()


#-- 'DockWindowHandler' interface implementation -------------------------

    #-------------------------------------------------------------------------
    #  Returns whether or not a specified object can be inserted into the view:
    #-------------------------------------------------------------------------

    def can_drop(self, info, object):
        """ Can the specified object be inserted into the view?
        """
        from pyface.dock.api import DockControl

        if isinstance(object, DockControl):
            return self.can_import(info, object.export)

        drop_class = info.ui.view.drop_class
        return ((drop_class is not None) and
                isinstance(object, drop_class))

    #-------------------------------------------------------------------------
    #  Returns whether or not a specified external view category can be
    #  imported:
    #-------------------------------------------------------------------------

    def can_import(self, info, category):
        return (category in info.ui.view.imports)

    #-------------------------------------------------------------------------
    #  Returns the DockControl object for a specified object:
    #-------------------------------------------------------------------------

    def dock_control_for(self, info, parent, object):
        """ Returns the DockControl object for a specified object.
        """
        from pyface.dock.api import IDockable, DockControl
        from .dockable_view_element import DockableViewElement

        try:
            name = object.name
        except:
            try:
                name = object.label
            except:
                name = ''
        if len(name) == 0:
            name = user_name_for(object.__class__.__name__)

        image = None
        export = ''
        if isinstance(object, DockControl):
            dock_control = object
            image = dock_control.image
            export = dock_control.export
            dockable = dock_control.dockable
            close = dockable.dockable_should_close()
            if close:
                dock_control.close(force=True)

            control = dockable.dockable_get_control(parent)

            # If DockControl was closed, then reset it to point to the new
            # control:
            if close:
                dock_control.set(control=control,
                                 style=parent.owner.style)
                dockable.dockable_init_dockcontrol(dock_control)
                return dock_control

        elif isinstance(object, IDockable):
            dockable = object
            control = dockable.dockable_get_control(parent)
        else:
            ui = object.get_dockable_ui(parent)
            dockable = DockableViewElement(ui=ui)
            export = ui.view.export
            control = ui.control

        dc = DockControl(control=control,
                         name=name,
                         export=export,
                         style=parent.owner.style,
                         image=image,
                         closeable=True)

        dockable.dockable_init_dockcontrol(dc)

        return dc

    #-------------------------------------------------------------------------
    #  Creates a new view of a specified control:
    #-------------------------------------------------------------------------

    def open_view_for(self, control, use_mouse=True):
        """ Creates a new view of a specified control.
        """
        from pyface.dock.api import DockWindowShell

        DockWindowShell(control, use_mouse=use_mouse)

    #-------------------------------------------------------------------------
    #  Handles a DockWindow becoming empty:
    #-------------------------------------------------------------------------

    def dock_window_empty(self, dock_window):
        """ Handles a DockWindow becoming empty.
        """
        if dock_window.auto_close:
            dock_window.control.GetParent.Destroy()

#-- HasTraits overrides: -------------------------------------------------

    #-------------------------------------------------------------------------
    #  Edits the object's traits: (Overrides HasTraits)
    #-------------------------------------------------------------------------

    def edit_traits(self, view=None, parent=None, kind=None,
                    context=None, handler=None, id='',
                    scrollable=None, **args):
        """ Edits the object's traits.
        """
        if context is None:
            context = self

        if handler is None:
            handler = self

        return self.trait_view(view).ui(
            context,
            parent,
            kind,
            self.trait_view_elements(),
            handler,
            id,
            scrollable,
            args)

    #-------------------------------------------------------------------------
    #  Configure the object's traits (Overrides HasTraits):
    #-------------------------------------------------------------------------

    def configure_traits(self, filename=None, view=None,
                         kind=None, edit=True,
                         context=None, handler=None,
                         id='', scrollable=None, **args):
        """ Configures the object's traits.
        """
        return super(HasPrivateTraits, self).configure_traits(
            filename, view, kind, edit, context, handler or self, id,
            scrollable, **args)

#-- Private Methods: -----------------------------------------------------

    #-------------------------------------------------------------------------
    #  Handles an 'Undo' change request:
    #-------------------------------------------------------------------------

    def _on_undo(self, info):
        """ Handles an "Undo" change request.
        """
        if info.ui.history is not None:
            info.ui.history.undo()

    #-------------------------------------------------------------------------
    #  Handles a 'Redo' change request:
    #-------------------------------------------------------------------------

    def _on_redo(self, info):
        """ Handles a "Redo" change request.
        """
        if info.ui.history is not None:
            info.ui.history.redo()

    #-------------------------------------------------------------------------
    #  Handles a 'Revert' all changes request:
    #-------------------------------------------------------------------------

    def _on_revert(self, info):
        """ Handles a "Revert all changes" request.
        """
        if info.ui.history is not None:
            info.ui.history.revert()
            self.revert(info)

    #-------------------------------------------------------------------------
    #  Handles a 'Close' request:
    #-------------------------------------------------------------------------

    def _on_close(self, info):
        """ Handles a "Close" request.
        """
        if (info.ui.owner is not None) and self.close(info, True):
            info.ui.owner.close()

#-------------------------------------------------------------------------
#  Default handler:
#-------------------------------------------------------------------------

_default_handler = Handler()


def default_handler(handler=None):
    """ Returns the global default handler.

    If *handler* is an instance of Handler, this function sets it as the
    global default handler.
    """
    global _default_handler

    if isinstance(handler, Handler):
        _default_handler = handler
    return _default_handler

#-------------------------------------------------------------------------
#  'Controller' class:
#-------------------------------------------------------------------------


class Controller(Handler):
    """ Defines a handler class which provides a view and controller for a
        specified model.

        This class is used when implementing a standard MVC-based design. The
        **model** trait contains most, if not all, of the data being viewed,
        and can be referenced in a Controller instance's View definition using
        unadorned trait names. (e.g., ``Item('name')``).
    """

    #-- Trait Definitions ----------------------------------------------------

    # The model this handler defines a view and controller for
    model = Instance(HasTraits)

    # The Info object associated with the controller
    info = Instance(UIInfo)

    #-- HasTraits Method Overrides -------------------------------------------

    def __init__(self, model=None, **metadata):
        """ Initializes the object and sets the model (if supplied).
        """
        super(Controller, self).__init__(**metadata)
        if model is not None:
            self.model = model

    def trait_context(self):
        """ Returns the default context to use for editing or configuring
            traits.
        """
        return {'object': self.model, 'controller': self, 'handler': self}

    #-- Handler Method Overrides ---------------------------------------------

    #-------------------------------------------------------------------------
    #  Informs the handler what the UIInfo object for a View will be:
    #-------------------------------------------------------------------------

    def init_info(self, info):
        """ Informs the handler what the UIInfo object for a View will be.
        """
        self.info = info

#-------------------------------------------------------------------------
#  'ModelView' class:
#-------------------------------------------------------------------------


class ModelView(Controller):
    """ Defines a handler class which provides a view and controller for a
        specified model.

        This class is useful when creating a variant of the standard MVC-based
        design. A subclass of ModelView reformulates a number of traits on
        its **model** object as properties on the ModelView subclass itself,
        usually in order to convert them into a more user-friendly format. In
        this design, the ModelView subclass supplies not only the view and
        the controller, but also, in effect, the model (as a set of properties
        wrapped around the original model). Because of this, the ModelView
        context dictionary specifies the ModelView instance itself as the
        special *object* value, and assigns the original model object as the
        *model* value. Thus, the traits of the ModelView object can be referenced
        in its View definition using unadorned trait names.
    """

    #-- HasTraits Method Overrides -------------------------------------------

    def trait_context(self):
        """ Returns the default context to use for editing or configuring
            traits.
        """
        return {'object': self, 'handler': self, 'model': self.model}

#-------------------------------------------------------------------------
#  'ViewHandler' class:
#-------------------------------------------------------------------------


class ViewHandler(Handler):

    pass



""" Context adapter factory for Python tuple. """


# Enthought library imports.
from apptools.naming.api import ContextAdapterFactory

# Local imports.
from tuple_context_adapter import TupleContextAdapter


class TupleContextAdapterFactory(ContextAdapterFactory):
    """ Context adapter factoryfor Python tuples. """

    #### 'ContextAdapterFactory' interface ####################################

    # The type of object that we adapt.
    adaptee_class = tuple

    ###########################################################################
    # Protected 'AbstractAdapterFactory' interface.
    ###########################################################################

    def _adapt(self, adaptee, target_class, environment, context):
        """ Returns an adapter that adapts an object to the target class. """

        adapter = TupleContextAdapter(
            adaptee     = adaptee,
            environment = environment,
            context     = context
        )

        return adapter

#### EOF ######################################################################



#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   01/10/2006
#
#------------------------------------------------------------------------------

""" Defines array editors for the WX user interface toolkit.
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------

# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
# compatibility. The class has been moved to the
# traitsui.editors.array_editor file.
from traitsui.editors.array_editor \
    import SimpleEditor as BaseSimpleEditor, ToolkitEditorFactory

from editor \
    import Editor

#-------------------------------------------------------------------------
#  'SimpleEditor' class:
#-------------------------------------------------------------------------


class SimpleEditor(BaseSimpleEditor, Editor):
    """ Simple style of editor for arrays.
    """
    # FIXME: This class has been re-defined here simply so it inherits from the
    # wx Editor class.
    pass

#-------------------------------------------------------------------------
#  'ReadonlyEditor' class:
#-------------------------------------------------------------------------


class ReadonlyEditor(SimpleEditor):

    # Set the value of the readonly trait.
    readonly = True

### EOF #######################################################################



""" The preferences manager. """


# Enthought library imports.
from traits.api import HasTraits, Instance, List, Property, \
    Any, Bool, Dict
from traitsui.api import Handler, HSplit, Item, TreeEditor
from traitsui.api import TreeNode, View, HTMLEditor
from traitsui.menu import Action

# Local imports.
from preferences_node import PreferencesNode
from preferences_page import PreferencesPage

# fixme: This is part of the attempt to allow developers to use non-Traits UI
# preferences pages. It doesn't work yet!
##from widget_editor import WidgetEditor


# A tree editor for preferences nodes.
tree_editor = TreeEditor(
    nodes = [
        TreeNode(
            node_for  = [PreferencesNode],
            auto_open = False,
            children  = 'children',
            label     = 'name',
            rename    = False,
            copy      = False,
            delete    = False,
            insert    = False,
            menu      = None,
        ),
    ],

    editable   = False,
    hide_root  = True,
    selected   = 'selected_node',
    show_icons = False
)


class PreferencesHelpWindow(HasTraits):
    """ Container class to present a view with string info. """

    def traits_view(self):
        """ Default view to show for this class. """
        args = []
        kw_args = {'title' : 'Preferences Page Help',
                   'buttons' : ['OK'],
                   'width' : 800,
                   'height' : 800,
                   'resizable' : True,
                   'id' : 'apptools.preferences.ui.preferences_manager.help'}
        to_show = {}

        for name, trait_obj in self.traits().items():
            if name != 'trait_added' and name != 'trait_modified':
                to_show[name] = trait_obj.help
        for name in to_show:
                args.append(Item(name,
                                 style='readonly',
                                 editor=HTMLEditor()
                                 ))

        view = View(*args, **kw_args)
        return view


class PreferencesManagerHandler(Handler):
    """ The traits UI handler for the preferences manager. """

    model = Instance(HasTraits)

    ###########################################################################
    # 'Handler' interface.
    ###########################################################################

    def apply(self, info):
        """ Handle the **Apply** button being clicked. """

        info.object.apply()
        return

    def init(self, info):
        """ Initialize the controls of a user interface. """

        # Select the first node in the tree (if there is one).
        self._select_first_node(info)

        return super(PreferencesManagerHandler, self).init(info)


    def close(self, info, is_ok):
        """ Close a dialog-based user interface. """

        if is_ok:
            info.object.apply()

        return super(PreferencesManagerHandler, self).close(info, is_ok)


    def preferences_help(self, info):
        """ Custom preferences help panel. The Traits help doesn't work."""
        current_page = self.model.selected_page
        to_show = {}
        for trait_name, trait_obj in current_page.traits().items():
            if hasattr(trait_obj, 'show_help') and trait_obj.show_help:
                to_show[trait_name] = trait_obj.help

        help_obj = PreferencesHelpWindow(**to_show)
        help_obj.edit_traits(kind='livemodal')
        return


    ###########################################################################
    # Private interface.
    ###########################################################################

    def _select_first_node(self, info):
        """ Select the first node in the tree (if there is one). """

        root = info.object.root

        if len(root.children) > 0:
            node = root.children[0]
            info.object.selected_page = node.page

        return


class PreferencesManager(HasTraits):
    """ The preferences manager. """

    # All of the preferences pages known to the manager.
    pages = List(PreferencesPage)

    # The root of the preferences node tree.
    root = Property(Instance(PreferencesNode))

    # The preferences node currently selected in the tree.
    selected_node = Instance(PreferencesNode)

    # The preferences associated with the currently selected preferences node.
    selected_page = Instance(PreferencesPage)

    # Should the custom Info button be shown?  If this is True, then an
    # Info button is shown that pops up a trait view with an HTML entry
    # for each trait of the *selected_page* with the metadata 'show_help'
    # set to True.
    show_help = Bool(False)

    # Should the Apply button be shown?
    show_apply = Bool(False)

    #### Traits UI views ######################################################

    def traits_view(self):
        """ Default traits view for this class. """

        help_action = Action(name = 'Info', action = 'preferences_help')

        buttons = ['OK', 'Cancel']

        if self.show_apply:
            buttons = ['Apply'] + buttons
        if self.show_help:
            buttons = [help_action] + buttons


        # A tree editor for preferences nodes.
        tree_editor = TreeEditor(
            nodes = [
                TreeNode(
                    node_for  = [PreferencesNode],
                    auto_open = False,
                    children  = 'children',
                    label     = 'name',
                    rename    = False,
                    copy      = False,
                    delete    = False,
                    insert    = False,
                    menu      = None,
                ),
            ],
            on_select  = self._selection_changed,
            editable   = False,
            hide_root  = True,
            selected   = 'selected_node',
            show_icons = False
        )

        view = View(
            HSplit(
                Item(
                    name       = 'root',
                    editor     = tree_editor,
                    show_label = False,
                    width      = 250,
                ),

                Item(
                    name       = 'selected_page',
                    #editor     = WidgetEditor(),
                    show_label = False,
                    width      = 450,
                    style      = 'custom',
                ),
            ),

            buttons   = buttons,
            handler   = PreferencesManagerHandler(model=self),
            resizable = True,
            title     = 'Preferences',
            width     = .3,
            height    = .3,
            kind      = 'modal'
        )
        self.selected_page = self.pages[0]
        return view

    ###########################################################################
    # 'PreferencesManager' interface.
    ###########################################################################

    #### Trait properties #####################################################

    def _get_root(self):
        """ Property getter. """

        # Sort the pages by the length of their category path. This makes it
        # easy for us to create the preference hierarchy as we know that all of
        # a node's ancestors will have already been created.
        def sort_key(a):
            # We have the guard because if the category is the empty string
            # then split will still return a list containing one item (and not
            # the empty list).
            if len(a.category) == 0:
                len_a = 0

            else:
                len_a = len(a.category.split('/'))

            return len_a

        self.pages.sort(key=sort_key)

        # Create a corresponding preference node hierarchy (the root of the
        # hierachy is NOT displayed in the preference dialog).
        #
        # fixme: Currently we have to create a dummy page for the root node
        # event though the root does not get shown in the tree!
        root_page = PreferencesPage(name='Root', preferences_path='root')
        root      = PreferencesNode(page=root_page)

        for page in self.pages:
            # Get the page's parent node.
            parent = self._get_parent(root, page)

            # Add a child node representing the page.
            parent.append(PreferencesNode(page=page))

        return root

    #### Trait change handlers ################################################

    def _selection_changed(self, new_selection):
        self.selected_node = new_selection


    def _selected_node_changed(self, new):
        """ Static trait change handler. """
        if self.selected_node:
            self.selected_page = self.selected_node.page

        return

    #### Methods ##############################################################

    def apply(self):
        """ Apply all changes made in the manager. """

        for page in self.pages:
            page.apply()

        return

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _get_parent(self, root, page):
        """ Return the page's parent preference node. """

        parent = root

        if len(page.category) > 0:
            components = page.category.split('/')
            for component in components:
                parent = parent.lookup(component)

        return parent

#### EOF ######################################################################



# Standard library imports
from cStringIO import StringIO
import logging
import os
import zipfile

# Enthought library imports
from pyface.workbench.api import View as WorkbenchView
from traits.api import Any, Callable, HasTraits, Instance, List, \
    Property, Undefined, on_trait_change

root_logger = logging.getLogger()
logger = logging.getLogger(__name__)


class LoggerService(HasTraits):
    """ The persistent service exposing the Logger plugin's API.
    """

    # The Envisage application.
    application = Any()

    # The logging Handler we use.
    handler = Any()

    # Our associated LoggerPreferences.
    preferences = Any()

    # The view we use.
    plugin_view = Instance(WorkbenchView)

    # Contributions from other plugins.
    mail_files = Property(List(Callable))

    def save_preferences(self):
        """ Save the preferences.
        """
        self.preferences.preferences.save()

    def whole_log_text(self):
        """ Return all of the logged data as formatted text.
        """
        lines = [ self.handler.format(rec) for rec in self.handler.get() ]
        # Ensure that we end with a newline.
        lines.append('')
        text = '\n'.join(lines)
        return text

    def create_email_message(self, fromaddr, toaddrs, ccaddrs, subject,
                             priority, include_userdata=False, stack_trace="",
                             comments="", include_environment=True):
        """ Format a bug report email from the log files.
        """
        from email.mime.application import MIMEApplication
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText

        message = MIMEMultipart()
        message['Subject'] = "%s [priority=%s]" % (subject, priority)
        message['To'] = ', '.join(toaddrs)
        message['Cc'] = ', '.join(ccaddrs)
        message['From'] = fromaddr
        message.preamble = 'You will not see this in a MIME-aware mail ' \
            'reader.\n'
        message.epilogue = ' ' # To guarantee the message ends with a newline

        # First section is simple ASCII data ...
        m = []
        m.append("Bug Report")
        m.append("==============================")
        m.append("")

        if len(comments) > 0:
            m.append("Comments:")
            m.append("========")
            m.append(comments)
            m.append("")

        if len(stack_trace) > 0:
            m.append("Stack Trace:")
            m.append("===========")
            m.append(stack_trace)
            m.append("")

        msg = MIMEText('\n'.join(m))
        message.attach(msg)

        # Include the log file ...
        logtext = self.whole_log_text()
        msg = MIMEText(logtext)
        msg.add_header('Content-Disposition', 'attachment',
            filename='logfile.txt')
        message.attach(msg)

        # Include the environment variables ...
        # FIXME: ask the user, maybe?
        if include_environment:
            # Transmit the user's environment settings as well.  Main purpose is
            # to work out the user name to help with following up on bug reports
            # and in future we should probably send less data.
            entries = []
            for key, value in sorted(os.environ.items()):
                entries.append('%30s : %s\n' % (key, value))

            msg = MIMEText(''.join(entries))
            msg.add_header('Content-Disposition', 'attachment',
                filename='environment.txt')
            message.attach(msg)

        if include_userdata and len(self.mail_files) != 0:
            f = StringIO()
            zf = zipfile.ZipFile(f, 'w')
            for mf in self.mail_files:
                mf(zf)
            zf.close()

            msg = MIMEApplication(f.getvalue())
            msg.add_header('Content-Disposition', 'attachment',
                filename='userdata.zip')
            message.attach(msg)

        return message

    def send_bug_report(self, smtp_server, fromaddr, toaddrs, ccaddrs, message):
        """ Send a bug report email.
        """
        try:
            import smtplib
            logger.debug("Connecting to: %s" % smtp_server)
            server = smtplib.SMTP(host=smtp_server)
            logger.debug("Connected: %s" % server)
            #server.set_debuglevel(1)
            server.sendmail(fromaddr, toaddrs + ccaddrs, message.as_string())
            server.quit()
        except Exception, e:
            logger.exception("Problem sending error report")

    #### Traits stuff #########################################################

    def _get_mail_files(self):
        return self.application.get_extensions(
            'apptools.logger.plugin.mail_files')

    @on_trait_change('preferences.level_')
    def _level_changed(self, new):
        if (new is not None and new is not Undefined and
            self.handler is not None):
            root_logger.setLevel(self.preferences.level_)
            self.handler.setLevel(self.preferences.level_)



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" The base class for all state factories. """


# Enthought library imports.
from traits.api import HasPrivateTraits


class StateFactory(HasPrivateTraits):
    """ The base class for all state factories.

    A state factory accepts an object and returns some data representing the
    object that is suitable for storing in a particular context.

    """

    ###########################################################################
    # 'StateFactory' interface.
    ###########################################################################

    def get_state_to_bind(self, obj, name, context):
        """ Returns the state of an object for binding.

        Returns None if the factory cannot create the state (ie. it does not
        recognise the object passed to it).

        """

        raise NotImplementedError

### EOF #######################################################################



#!/usr/bin/env python
"""
write_kmz.py

module that supports writng kml / kmz for trajectory files
"""
from mako.template import Template
import zipfile, base64
import os,sys
import kml_templates

def write_kmz(kmz_filename, times, best_guess_positions):
    """
    write a kmz file

    at this point, only "best guess" particles are supported

    :param kmz_filename: filename of resulting kmz file

    :param times: list of datetimes of the time for each time step

    :param best_guess_positions: lat-lon of particle positions.

    """
    basefilename = kmz_filename.rstrip(".kmz")
    kmz_filename = basefilename + ".kmz"

    mytemplate = Template(kml_templates.LE_Template)

    print "rendering: %s"%kmz_filename

    for i, dt in enumerate(times):
        bestGuessLEs = best_guess_positions[i]
        description = "A description as a placeholder"
        spillInfo = "Sample Spill info"
        
        fromLogo = "" # turn this off for now

        #LEs_Floating = ReadMoss.FilterLEsByStatus(bestGuessLEs,"INWATER")
        #LEs_Beached = ReadMoss.FilterLEsByStatus(bestGuessLEs,"ONBEACH")
        
        #uncertaintyLEs_Floating = ReadMoss.FilterLEsByStatus(uncertaintyLEs,"INWATER")
        #uncertaintyLEs_Beached = ReadMoss.FilterLEsByStatus(uncertaintyLEs,"ONBEACH")
        
   
        mytemplate = Template(mkl_templates.LE_Template)
    
        # ( Z = zulu, or can have +-hh:mm for conversion to UTC
        #mossTime = spillInfo["VALIDFOR"] #  example: 15:00, 08/17/09
        #gmtOffset = 0 # code goes here
        beginTime = dt.isoformat()
        endTime = None
        #print "mossTime %s\n beginTime %s" %(mossTime,beginTime)
        
    
        altitude = 0.1 # raise the best guess LEs up above the water so that they are always on top of the uncertainty LEs
        LE_Sets = [
                   (LEs_Floating,"%s - %s" %(mossTime,"Floating Splots (Best Guess)"),"#YellowDotIcon",altitude),
                   (LEs_Beached,"%s - %s" %(mossTime,"Beached Splots (Best Guess)"),"#YellowXIcon",altitude)
                   ]
        
        # altitude = 0 # on the water surface
        # UncertaintyLE_Sets = [
        #            (uncertaintyLEs_Floating,"%s - %s" %(mossTime,"Uncertainty Floating Splots"),"#RedDotIcon",altitude),
        #            (uncertaintyLEs_Beached,"%s - %s" %(mossTime,"Uncertainty Beached Splots"),"#RedXIcon",altitude)
        #            ]
        
        # Polygon_Sets = [
        #                 (forecastHeavyPolygons,"%s - %s" %(mossTime,"Forecast Heavy"),"#HeavyStyle",altitude),
        #                (forecastMediumPolygons,"%s - %s" %(mossTime,"Forecast Medium"),"#MediumStyle",altitude),
        #                (forecastLightPolygons,"%s - %s" %(mossTime,"Forecast Light"),"#LightStyle",altitude),
        #                (uncertaintyPolygons,"%s - %s" %(mossTime,"Uncertainty"),"#UncertaintyStyle",altitude)                  
        #               ]
        
        seriesItem = MossSeriesItem(beginTime,endTime,LE_Sets)
        BestGuess_Series.append(seriesItem)
        
        seriesItem = MossSeriesItem(beginTime,endTime,UncertaintyLE_Sets)
        Uncertainty_Series.append(seriesItem)
       
        #seriesItem = MossSeriesItem(beginTime,endTime,Polygon_Sets)
        #Polygon_Series.append(seriesItem)
    
    # set the end times
    for timeSeries in [BestGuess_Series,Uncertainty_Series,Polygon_Series]:
        for i in range(0,len(timeSeries)-1):
            timeSeries[i].endTime = timeSeries[i+1].beginTime
        
    kmlstring = mytemplate.render(BestGuess_Series =BestGuess_Series, 
                                  Uncertainty_Series =Uncertainty_Series, 
                                  Polygon_Series =Polygon_Series, 
                                  name=Name,
                                  description = description,
                                  fromLogo = fromLogo)
                                   
    kmzfile = zipfile.ZipFile(outfilename, 'w', compression=zipfile.ZIP_DEFLATED)
    
    kmzfile.writestr('dot.png', base64.b64decode(kml_templates.DotData))
    kmzfile.writestr('x.png', base64.b64decode(kml_templates.XData))
    kmzfile.writestr('scalebar.png', base64.b64decode(kml_templates.ScaleBarData))
    
    kmzfile.writestr(Name+".kml", kmlstring)
    kmzfile.close() 
        
    if printDiagnostic: print kmlstring
    
    print "done" 





#------------------------------------------------------------------------------
#
#  Copyright (c) 2005, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: Enthought, Inc.
#
#------------------------------------------------------------------------------

""" Enthought pyface package component
"""

# Major package imports.
import wx

# Enthought library imports.
from traits.api import Bool, Enum, Instance, provides, Unicode

# Local imports.
from pyface.i_confirmation_dialog import IConfirmationDialog, MConfirmationDialog
from pyface.constant import CANCEL, YES, NO
from pyface.image_resource import ImageResource
from .dialog import Dialog


@provides(IConfirmationDialog)
class ConfirmationDialog(MConfirmationDialog, Dialog):
    """ The toolkit specific implementation of a ConfirmationDialog.  See the
    IConfirmationDialog interface for the API documentation.
    """


    #### 'IConfirmationDialog' interface ######################################

    cancel = Bool(False)

    default = Enum(NO, YES, CANCEL)

    image = Instance(ImageResource)

    message = Unicode

    informative = Unicode

    detail = Unicode

    no_label = Unicode

    yes_label = Unicode

    ###########################################################################
    # Protected 'IDialog' interface.
    ###########################################################################

    def _create_buttons(self, parent):
        sizer = wx.StdDialogButtonSizer()

        # 'YES' button.
        if self.yes_label:
            label = self.yes_label
        else:
            label = "Yes"

        self._yes = yes = wx.Button(parent, wx.ID_YES, label)
        if self.default == YES:
            yes.SetDefault()
        wx.EVT_BUTTON(parent, wx.ID_YES, self._on_yes)
        sizer.AddButton(yes)

        # 'NO' button.
        if self.no_label:
            label = self.no_label
        else:
            label = "No"

        self._no = no = wx.Button(parent, wx.ID_NO, label)
        if self.default == NO:
            no.SetDefault()
        wx.EVT_BUTTON(parent, wx.ID_NO, self._on_no)
        sizer.AddButton(no)

        if self.cancel:
            # 'Cancel' button.
            if self.no_label:
                label = self.cancel_label
            else:
                label = "Cancel"

            self._cancel = cancel = wx.Button(parent, wx.ID_CANCEL, label)
            if self.default == CANCEL:
                cancel.SetDefault()

            wx.EVT_BUTTON(parent, wx.ID_CANCEL, self._wx_on_cancel)
            sizer.AddButton(cancel)

        sizer.Realize()
        return sizer

    def _create_dialog_area(self, parent):
        panel = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)

        # The image.
        if self.image is None:
            image_rc = ImageResource('warning')
        else:
            image_rc = self.image

        image = wx.StaticBitmap(panel, -1, image_rc.create_bitmap())
        sizer.Add(image, 0, wx.EXPAND | wx.ALL, 10)

        # The message.
        if self.informative:
            message = self.message + '\n\n' + self.informative
        else:
            message = self.message
        message = wx.StaticText(panel, -1, message)
        sizer.Add(message, 1, wx.EXPAND | wx.TOP, 15)

        # Resize the panel to match the sizer.
        sizer.Fit(panel)

        return panel

    ###########################################################################
    # Private interface.
    ###########################################################################

    #### wx event handlers ####################################################

    def _on_yes(self, event):
        """ Called when the 'Yes' button is pressed. """

        self.control.EndModal(wx.ID_YES)

    def _on_no(self, event):
        """ Called when the 'No' button is pressed. """

        self.control.EndModal(wx.ID_NO)

#### EOF ######################################################################



""" Tests for the trait documenter. """

import StringIO
import sys
import tokenize

from traits.testing.unittest_tools import unittest


def _sphinx_present():
    try:
        import sphinx  # noqa
    except ImportError:
        return False

    return True


def _python_version_is_32():
    return sys.version_info[:2] == (3, 2)


# Skipping for python 3.2 because sphinx does not work on it.
@unittest.skipIf(not _sphinx_present() or _python_version_is_32(),
                 "Sphinx not available. Cannot test documenter")
class TestTraitDocumenter(unittest.TestCase):
    """ Tests for the trait documenter. """

    def setUp(self):
        self.source = """
    depth_interval = Property(Tuple(Float, Float),
                              depends_on="_depth_interval")
"""
        string_io = StringIO.StringIO(self.source)
        tokens = tokenize.generate_tokens(string_io.readline)
        self.tokens = tokens

    def test_get_definition_tokens(self):

        from traits.util.trait_documenter import _get_definition_tokens

        definition_tokens = _get_definition_tokens(self.tokens)

        # Check if they are correctly untokenized. This should not raise.
        string = tokenize.untokenize(definition_tokens)

        self.assertEqual(self.source.rstrip(), string)

if __name__ == '__main__':
    unittest.main()

# ## EOF ######################################################################



import json
import os
import matplotlib.colors as colors

cnames = colors.cnames
# Add the single letter colors.
for name, rgb in colors.ColorConverter.colors.iteritems():
    hex_ = colors.rgb2hex(rgb)
    cnames[name] = hex_


def points(package_dir, fn, params):
    '''
    params is a dict with the following keys:
        site_name
        event (can be None)
        title
        shape_zipfilename
        meta_data
        attachment_file - just set up to do one at the moment
        color - matplotlib named colors (e.g. "r" or "red")
        folder_path - a list with the hierarchy of subfolders in TOC
    '''
    try:
        color = cnames[params['color']]
    except:
        color = '#000000'

    sites = {}
    sites['dwh'] = {'site_id': 1, 'zoom': 6, 'longitude': -90.42, 'latitude': 28.03}
    sites['arctic'] = {'site_id': 1, 'zoom': 5, 'longitude': -150.00, 'latitude': 70.00}
    sites['atlantic'] = {'site_id': 2, 'zoom': 7, 'longitude': -74.10, 'latitude': 	38.94}
    sites['caribbean'] = {'site_id': 3, 'zoom': 6, 'longitude': -65.44, 'latitude': 18.20}
    sites['greatlakes'] = {'site_id': 9, 'zoom': 1, 'longitude': -83.44, 'latitude': 45.60}
    sites['gulfofmexico'] = {'site_id': 4, 'zoom': 4, 'longitude': -90.42, 'latitude': 28.03}
    sites['northwest'] = {'site_id': 6, 'zoom': 6, 'longitude': -125.00, 'latitude': 47.00}
    sites['pacific'] = {'site_id': 7, 'zoom': 4, 'longitude': -178.00, 'latitude': 11.00}
    sites['southwest'] = {'site_id': 8, 'zoom': 6, 'longitude': -122.00, 'latitude': 37.00}


    filename = os.path.join(package_dir, 'layers', fn + '.json')
    f = file(filename, 'w')

    layer_info = {
      "layer_type": "wms internal",
      "folder_path": ' > '.join(params['folder_path']),
      "additional_metainfo": params['metadata'],
      "ephemeral": False,
      "sensitivity": "Responders",
      "background_layer": False,
      "legend_annotation": None,
      "applicability": [
        {
          "event": params['event'],
          "site": {
            "site_path": "/" + params['site_name'],
            "site_name": params['site_name'],
            "site_id": sites[params['site_name']]['site_id'],
            "zoom": sites[params['site_name']]['zoom'],
            "longitude": sites[params['site_name']]['longitude'],
            "extent": None,
            "latitude": sites[params['site_name']]['latitude'],
            "baselayer": 16
          }
        }
      ],
      "tilecache": True,
      "modified_by": None,
      "layer_names": None,
      "refresh_rate": 0,
      "title": params['title'],  # name of layer as it appears in TOC
      "single_tile": False,
      "created_by": None,
      "legendgraphic": None,
      "marker_icon": None,
      "external_proj": None,
      "strokecolor": None,
      "opacity": 1,
      "mapfile_layer": {
        "layer_type": "point",
        "opacity": 100,
        "sort_field": "gid",
        "classitem": "status",
        "labelitem": None,
        "date_modified": None,
        "shapefile": {
          "file": "file://source_files/" + params['shape_zipfilename'],
          "has_hotlinks": False,
          "name": params['shape_zipfilename'].split('.')[0], #a "friendly" display name for the shapefile
          "description": None,
          "created_by": None,
          "keywords": None,
          "allow_download": True,
          "timezone_fields": None,
          "content_type": "application/zip",
          "srid": 4326
        },
        "time_column": None,
        "maxscaledenom": None,
        "created_by": None,
        "labelminscaledenom": None,
        "layer_classes": [
          {
            "styles": [
              {
                "angle_field": None,
                "angle": None,
                "font_character": None,
                "map_layer_class": 101964,
                "ordering": 1,
                "symbol": {
                  "style_text": None,
                  "symbol_def": "NAME 'cross'\nTYPE vector\nPOINTS\n 2 0\n 2 4\n -99 -99\n 0 2\n 4 2\nEND\n",
                  "symbol_desc": "Cross",
                  "symbol_name": "cross"
                },
                "size_field": None,
                "color": color,
                "style_width": None,
                "offset_x": None,
                "offset_y": None,
                "outlinesymbol": None,
                "outlinecolor": None,
                "width_field": None,
                "style_size": 5
              }
            ],
            "name": "Beached",
            "ordering": 2,
            "expression_type": {
              "expression_type": "V",
              "description": "Exact Value"
            },
            "labels": [
              
            ],
            "expression": "3"
          },
          {
            "styles": [
              {
                "angle_field": None,
                "angle": None,
                "font_character": None,
                "map_layer_class": 101965,
                "ordering": 1,
                "symbol": {
                  "style_text": None,
                  "symbol_def": "NAME 'filledcircle'\nTYPE ellipse\nFILLED True\nPOINTS\n 1 1\nEND\n",
                  "symbol_desc": "Circle (Filled)",
                  "symbol_name": "filledcircle"
                },
                "size_field": None,
                "color": color,
                "style_width": None,
                "offset_x": None,
                "offset_y": None,
                "outlinesymbol": None,
                "outlinecolor": None,
                "width_field": None,
                "style_size": 5
              }
            ],
            "name": "Floating",
            "ordering": 3,
            "expression_type": {
              "expression_type": "V",
              "description": "Exact Value"
            },
            "labels": [

            ],
            "expression": "2"
          }
        ],
        "sort_order": None,
        "labelmaxscaledenom": None,
        "template": False,
        "modified_by": None,
        "layer_desc": 'Styling for trajectory points file',
        "minscaledenom": None,
        "layer_name": "trajectory_points_layer"
      },
      "metadata_url": None,
      "visibility": None,
      "strokewidth": None,
      "proxy": False,
      "animate_script": None,
      "legend": None,
      "uuid": None,
      "gutter": None,
      "graphicname": None,
      "date_modified": None,
      "url": None,
      "metadata_path": None,
      "pointradius": None,
      "fillcolor": None
    }

    if params['attachment_file'] is not None:
        layer_info["attachments"] = [
            {"ordering": 1,
             "attachment": "file://attachments/" + params['attachment_file'],
             "content_type": "application/gif"}
        ]

    json.dump(layer_info, f, indent=2)
    f.close()


def contours(package_dir, fn, params):
    '''
    params is a dict with the following keys:
        site_name
        event (can be None)
        title
        shape_zipfilename
        meta_data
        attachment_file - just set up to do one at the moment
        color - matplotlib named colors (e.g. "r" or "red")
        folder_path - a list with the hierarchy of subfolders in TOC
        uncertain - just has to have this param for uncertainty layer
    '''

    sites = {}
    sites['dwh'] = {'site_id':1, 'zoom': 6, 'longitude': -90.42, 'latitude': 28.03}
    sites['arctic'] = {'site_id': 1, 'zoom': 5, 'longitude': -150.00, 'latitude': 70.00}
    sites['atlantic'] = {'site_id': 2, 'zoom': 7, 'longitude': -74.10, 'latitude': 38.94}
    sites['caribbean'] = {'site_id': 3, 'zoom': 6, 'longitude': -65.44, 'latitude': 18.20}
    sites['greatlakes'] = {'site_id': 9, 'zoom': 1, 'longitude': -83.44, 'latitude': 45.60}
    sites['gulfofmexico'] = {'site_id': 4, 'zoom': 4, 'longitude': -90.42, 'latitude': 28.03}
    sites['northwest'] = {'site_id': 6, 'zoom': 6, 'longitude': -125.00, 'latitude': 47.00}
    sites['pacific'] = {'site_id': 7, 'zoom': 4, 'longitude': -178.00, 'latitude': 11.00}
    sites['southwest'] = {'site_id': 8, 'zoom': 6, 'longitude': -122.00, 'latitude': 37.00}

    filename = os.path.join(package_dir,'layers',fn + '.json')
    f = file(filename,'w')

    if params.has_key('Uncertain'):

        layer_classes = [
              {
                "styles": [
                  {
                    "angle_field": None,
                    "angle": None,
                    "font_character": None,
                    "map_layer_class": 102054,
                    "ordering": 1,
                    "symbol": None,
                    "size_field": None,
                    "color": None,
                    "style_width": None,
                    "offset_x": None,
                    "offset_y": None,
                    "outlinesymbol": None,
                    "outlinecolor": "#bf0000",
                    "width_field": None,
                    "style_size": None
                  }
                ],
                "name": "Uncertainty Boundary",
                "ordering": 1,
                "expression_type": {
                  "expression_type": "V",
                  "description": "Exact Value"
                },
                "labels": [

                ],
                "expression": "Uncertainty"
              }
            ]

    elif params.has_key('SinglePoly'):

        layer_classes = [
              {
                "styles": [
                  {
                    "angle_field": None,
                    "angle": None,
                    "font_character": None,
                    "map_layer_class": 102031,
                    "ordering": 1,
                    "symbol": None,
                    "size_field": None,
                    "color": "#00ffff",
                    "style_width": None,
                    "offset_x": None,
                    "offset_y": None,
                    "outlinesymbol": None,
                    "outlinecolor": "#000000",
                    "width_field": None,
                    "style_size": None
                  }
                ],
                "name": "Most Probable Location",
                "ordering": 1,
                "expression_type": {
                  "expression_type": "V",
                  "description": "Exact Value"
                },
                "labels": [
                ],
                "expression": "Light"
              }
            ]
    else:
        layer_classes = [
           {
                "styles": [
                  {
                    "angle_field": None,
                    "angle": None,
                    "font_character": None,
                    "map_layer_class": 102030,
                    "ordering": 1,
                    "symbol": None,
                    "size_field": None,
                    "color": "#0000ff",
                    "style_width": None,
                    "offset_x": None,
                    "offset_y": None,
                    "outlinesymbol": None,
                    "outlinecolor": "#000000",
                    "width_field": None,
                    "style_size": None
                  }
                ],
                "name": "Heavy",
                "ordering": 2,
                "expression_type": {
                      "expression_type": "V",
                      "description": "Exact Value"
                },
                "labels": [
                ],
                "expression": "Heavy"
              },
              {
                "styles": [
                  {
                    "angle_field": None,
                    "angle": None,
                    "font_character": None,
                    "map_layer_class": 102029,
                    "ordering": 1,
                    "symbol": None,
                    "size_field": None,
                    "color": "#007fff",
                    "style_width": None,
                    "offset_x": None,
                    "offset_y": None,
                    "outlinesymbol": None,
                    "outlinecolor": "#000000",
                    "width_field": None,
                    "style_size": None
                  }
                ],
                "name": "Medium",
                "ordering": 3,
                "expression_type": {
                    "expression_type": "V",
                    "description": "Exact Value"
                },
                "labels": [
                ],
                "expression": "Medium"
              },
              {
                "styles": [
                  {
                    "angle_field": None,
                    "angle": None,
                    "font_character": None,
                    "map_layer_class": 102031,
                    "ordering": 1,
                    "symbol": None,
                    "size_field": None,
                    "color": "#00ffff",
                    "style_width": None,
                    "offset_x": None,
                    "offset_y": None,
                    "outlinesymbol": None,
                    "outlinecolor": "#000000",
                    "width_field": None,
                    "style_size": None
                  }
                ],
                "name": "Light",
                "ordering": 4,
                "expression_type": {
                  "expression_type": "V",
                  "description": "Exact Value"
                },
                "labels": [
                ],
                "expression": "Light"
              }
              ]
    layer_info = {
        "layer_type": "wms internal",
        "folder_path": ' > '.join(params['folder_path']),
        "additional_metainfo": params['metadata'],
        "ephemeral": False,
        "sensitivity": "Responders",
        "background_layer": False,
        "legend_annotation": None,
        "applicability": [
          {
            "event": params['event'],
            "site": {
                "site_path": "/" + params['site_name'],
                "site_name": params['site_name'],
                "site_id": sites[params['site_name']]['site_id'],
                "zoom": sites[params['site_name']]['zoom'],
                "longitude": sites[params['site_name']]['longitude'],
                "extent": None,
                "latitude": sites[params['site_name']]['latitude'],
                "baselayer": 16
            }
         }
      ],
      "tilecache": True,
      "modified_by": None,
      "layer_names": None,
      "refresh_rate": 0,
      "title": params['title'], #name of layer as it appears in TOC
      "single_tile": False,
      "created_by": None,
      "legendgraphic": None,
      "marker_icon": None,
      "external_proj": None,
      "strokecolor": None,
      "opacity": 1,
      "mapfile_layer": {
        "layer_type": "polygon",
        "opacity": 100,
        "sort_field": "type",
        "classitem": "type",
        "labelitem": None,
        "date_modified": None,
        "shapefile": {
          "file": "file://source_files/" + params['shape_zipfilename'],
          "has_hotlinks": False,
          "name": params['shape_zipfilename'].split('.')[0], #a "friendly" display name for the shapefile
          "description": None,
          "created_by": None,
          "keywords": None,
          "allow_download": True,
          "timezone_fields": None,
          "content_type": "application/zip",
          "srid": 4326
        },
        "time_column": None,
        "maxscaledenom": None,
        "created_by": None,
        "labelminscaledenom": None,
        "layer_classes": layer_classes,
        "sort_order": None,
        "labelmaxscaledenom": None,
        "template": False,
        "modified_by": None,
        "layer_desc": 'Styling for trajectory contours',
        "minscaledenom": None,
        "layer_name": "trajectory_contour_layer"
      },
      "metadata_url": None,
      "visibility": None,
      "strokewidth": None,
      "proxy": False,
      "animate_script": None,
      "legend": None,
      "uuid": None,
      "gutter": None,
      "graphicname": None,
      "date_modified": None,
      "url": None,
      "metadata_path": None,
      "pointradius": None,
      "fillcolor": None
    }

    if params['attachment_file'] is not None:
        layer_info["attachments"] = [
            {"ordering": 1,
             "attachment": "file://attachments/" + params['attachment_file'],
             "content_type": "application/gif"}
        ]

    json.dump(layer_info, f, indent=2)
    f.close()


""" Download manager using background threads
"""
import os
import time
import urllib2 as urllib2

import wx
import wx.lib.scrolledpanel as scrolled

from background_http import BaseRequest, BackgroundHttpMultiDownloader

import logging
log = logging.getLogger(__name__)


class DownloadURLRequest(BaseRequest):
    def __init__(self, url, path):
        BaseRequest.__init__(self)
        self.url = url
        self.is_skippable = False
#        self.blocksize = 16 * 1024
        self.blocksize = 1024
        self.expected_size = 0
        self.size = 0
        self.is_cancelled = False
        self.path = path

    def __str__(self):
        if not self.is_started:
            return "%s download not started" % (self.url)
        elif self.is_finished:
            if self.error is None:
                if self.data:
                    return "%s returned %d bytes in %s" % (self.url, self.size, self.path)
                else:
                    return "%s incomplete download, %d of %d bytes" % (self.url, self.size, self.expected_size)
            else:
                return "%s returned error: %s" % (self.url, self.error)
        else:
            return "%s downloading, %d of %d in %s" % (self.url, self.size, self.expected_size, self.path)

    def cancel(self):
        self.is_cancelled = True

    def get_data_from_server(self):
        try:
            request = urllib2.Request(self.url)
            response = urllib2.urlopen(request)
            headers = response.info()
            if "Content-Length" in headers:
                self.expected_size = int(headers['Content-Length'])
            finished = False
            tmp_path = self.path + ".download"
            with open(tmp_path, 'wb') as fh:
                while True:
                    chunk = response.read(self.blocksize)
                    if not chunk:
                        finished = True
                        break
                    if self.is_cancelled:
                        break
                    fh.write(chunk)
                    self.size += len(chunk)
                    print "size:", self.size
                    time.sleep(.1)
            if finished:
                try:
                    os.remove(self.path)
                except OSError:
                    pass
                os.rename(tmp_path, self.path)
                self.data = self.path

        except urllib2.URLError, e:
            self.error = e

        except ValueError, e:
            self.error = e

        except OSError, e:
            self.error = e


class RequestStatusControl(wx.Panel):
    border = 5

    def __init__(self, parent, req, **kwargs):
        wx.Panel.__init__(self, parent, -1, **kwargs)
        hbox = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(hbox)
        self.text = wx.StaticText(parent, -1, rec.url)
        hbox.Add(self.text, 0, flag=wx.EXPAND|wx.ALL, border=self.border)
        self.gauge = wx.Gauge(parent, -1)
        hbox.Add(self.gauge, 0, flag=wx.EXPAND|wx.ALL, border=self.border)
        self.cancel = wx.Button(parent, -1, "Cancel")
        hbox.Add(self.cancel, 0, flag=wx.EXPAND|wx.ALL, border=self.border)


class DownloadControl(scrolled.ScrolledPanel):
    """
    View of list of downloaded items
    """

    def __init__(self, parent, downloader, **kwargs):
        scrolled.ScrolledPanel.__init__(self, parent, -1, **kwargs)
        hbox = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(hbox)
        self.ShowScrollbars(wx.SHOW_SB_NEVER, wx.SHOW_SB_ALWAYS)
        self.SetupScrolling(scroll_x=False)
        hbox.Layout()
        self.Fit()
        self.downloader= downloader

    def add_request(self, req):
        rc = RequestStatusControl(self.GetParent(), req)
        self.GetSizer().Add(rc, 0, flag=wx.EXPAND)


if __name__ == "__main__":
    from background_http import BackgroundHttpDownloader

    class MyFrame(wx.Frame):
        def __init__(self, parent, id, title):
            wx.Frame.__init__(self, parent, id, title, wx.DefaultPosition, wx.DefaultSize)
            self.dlcontrol = DownloadControl(self, None)

    class MyApp(wx.App):
        def OnInit(self):
            self.dlframe = MyFrame(None, -1, 'Download Manager')
            self.dlframe.Show(True)
            self.dlframe.Centre()
            return True

    app = MyApp(0)
    downloader = BackgroundHttpDownloader()
    app.dlframe.dlcontrol.downloader = downloader
    app.MainLoop()

    downloader = BackgroundHttpDownloader()
    # downloader.send_request(URLRequest('http://www.python.org/'))
    # downloader.send_request(URLRequest('http://www.doughellmann.com/PyMOTW/contents.html'))
    req = DownloadURLRequest('http://playermissile.com', "index.html")
    downloader.send_request(req)
    print req
    # downloader.send_request(URLRequest('http://docs.python.org/release/2.6.8/_static/py.png'))
    # downloader.send_request(URLRequest('http://image.tmdb.org/t/p/w342/vpk4hLyiuI2SqCss0T3jeoYcL8E.jpg'))
    # downloader.send_request(URLRequest('hvvttp://playermissile.com'))
    first = True
    for i in range(10):
        print "STEP", i, req
        downloaded = downloader.get_finished()
        for url in downloaded:
            print 'FINISHED:', url
        # if i > 1 and first:
        #     downloader.send_request(URLRequest('http://www.python.org/images/python-logo.gif'))
        #     downloader.send_request(URLRequest('http://www.python.org/'))
        #     downloader.send_request(URLRequest('http://www.doughellmann.com/PyMOTW/contents.html'))
        #     downloader.send_request(URLRequest('http://playermissile.com'))
        #     first = False
        if i == 5:
            req.cancel()
        time.sleep(.1)

    downloader = None



#-------------------------------------------------------------------------
#
#  Copyright (c) 2008, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   07/26/2006
#
#-------------------------------------------------------------------------

""" Defines a completely empty editor, intended to be used as a spacer.
"""

#-------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------

from __future__ import absolute_import

from ..basic_editor_factory import BasicEditorFactory

from ..toolkit import toolkit_object

# Callable which returns the editor to use in the ui.


def null_editor(*args, **traits):
    return toolkit_object('null_editor:NullEditor')(*args, **traits)

#-------------------------------------------------------------------------
#  Create the editor factory object:
#-------------------------------------------------------------------------
NullEditor = BasicEditorFactory(klass=null_editor)

### EOF ---------------------------------------------------------------------




from traits.api import Bool, List, Str
from .date_editor import DateEditor


class CellFormat(object):
    """ Encapsulates some common visual attributes to set on the cells of a
    calendar widget.  All attributes default to None, which means that they
    will not override the existing values of the calendar widget.
    """

    italics = None
    bold = None
    underline = None

    # The color attributes should be strings representing color names,
    # from the list:
    #   red, green, blue, cyan, magenta, yellow, gray, white,
    #   darkRed, darkGreen, darkBlue, darkCyan, darkmagenta, darkYellow, darkGray,
    #   black, lightGray
    #
    # Alternatively, they can be a tuple of (R,G,B) values from 0-255.
    bgcolor = None
    fgcolor = None

    def __init__(self, **args):
        for key, val in args.items():
            setattr(self, key, val)


class ToolkitEditorFactory(DateEditor):
    """ A DateEditor that can show sets of dates in different styles.
    """

    # The name of a dictionary on the object that maps names to groups
    # (list/tuples) of datetime.date objects.  Each of these groups can be
    # styled using the **styles** dict.
    dates_trait = Str()

    # The name of a dictionary on the object that maps names of styles to
    # CellFormat objects.  The names used must match the names used in the
    # **dates** dict.
    styles_trait = Str()

    # Allow selection of arbitrary dates in the past.
    allow_past = Bool(True)

    # Allow selection of arbitrary dates in the future.
    allow_future = Bool(True)

    # A list of strings that will be offered as an alternative to specifying
    # an absolute date, and instead specify a relative date.
    relative_dates = List()

StyledDateEditor = ToolkitEditorFactory
